@@ -65,11 +65,12 @@ enum {
 };
 
 /* generic templates */
-GST_PAD_TEMPLATE_FACTORY (spider_src_factory,
+static GstStaticPadTemplate spider_src_factory =
+GST_STATIC_PAD_TEMPLATE (
   "src_%d",
   GST_PAD_SRC,
   GST_PAD_REQUEST,
-  NULL      /* no caps */
+  GST_STATIC_CAPS_ANY
 );
 
 /* standard GObject stuff */
@@ -152,7 +153,8 @@ gst_spider_class_init (GstSpiderClass *klass)
   gobject_class->get_property = gst_spider_get_property;
   gobject_class->dispose = gst_spider_dispose;
 
-  gst_element_class_add_pad_template (gstelement_class, GST_PAD_TEMPLATE_GET (spider_src_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&spider_src_factory));
   gst_element_class_set_details (gstelement_class, &gst_spider_details);
   
   gstelement_class->request_new_pad = GST_DEBUG_FUNCPTR(gst_spider_request_new_pad);
@@ -209,6 +211,7 @@ gst_spider_request_new_pad (GstElement *element, GstPadTemplate *templ, const gc
   gst_bin_add (GST_BIN (element), GST_ELEMENT (identity));
   
   returnpad = gst_element_add_ghost_pad (element, returnpad, padname);
+  g_free (padname);
   gst_spider_link_new (identity);
   GST_DEBUG ("successuflly created requested pad %s:%s", GST_DEBUG_PAD_NAME (returnpad));
   
@@ -586,13 +589,28 @@ gst_spider_plug_from_srcpad (GstSpiderConnection *conn, GstPad *srcpad)
   gboolean result = TRUE;
   GstSpider *spider = (GstSpider *) GST_OBJECT_PARENT (conn->src);
   GstElement *startelement = conn->current;
+  GstCaps *caps1;
+  GstCaps *caps2;
 
   g_assert ((GstElement *) GST_OBJECT_PARENT (srcpad) == conn->current);
   GST_DEBUG ("trying to plug from %s:%s to %s", 
 	     GST_DEBUG_PAD_NAME (srcpad), GST_ELEMENT_NAME (conn->src));
   
+  /* see if they match already */
+  if (gst_pad_link (srcpad, conn->src->sink)) {
+    GST_DEBUG ("%s:%s and %s:%s can link directly", 
+	       GST_DEBUG_PAD_NAME (srcpad), GST_DEBUG_PAD_NAME (conn->src->sink));
+    gst_pad_unlink (srcpad, conn->src->sink);
+    gst_spider_create_and_plug (conn, NULL);
+    return GST_PAD_LINK_OK;
+  }
+      
   /* find a path from src to sink */
-  plugpath = gst_autoplug_sp (gst_pad_get_caps (srcpad), gst_pad_get_caps (conn->src->sink), spider->factories);
+  caps1 = gst_pad_get_caps (srcpad);
+  caps2 = gst_pad_get_caps (conn->src->sink);
+  plugpath = gst_autoplug_sp (caps1, caps2, spider->factories);
+  gst_caps_free (caps1);
+  gst_caps_free (caps2);
   
   /* prints out the path that was found for plugging */
   /* g_print ("found path from %s to %s:\n", GST_ELEMENT_NAME (conn->current), GST_ELEMENT_NAME (conn->src));
@@ -662,7 +680,6 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   GST_LICENSE,
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN
 )
