@@ -27,35 +27,34 @@
 #include "gstspideridentity.h"
 #include "gstspider.h"
 
-GST_DEBUG_CATEGORY (gst_spider_identity_debug);
+GST_DEBUG_CATEGORY_STATIC (gst_spider_identity_debug);
 #define GST_CAT_DEFAULT gst_spider_identity_debug
 
-GstElementDetails gst_spider_identity_details = {
+static GstElementDetails gst_spider_identity_details = GST_ELEMENT_DETAILS (
   "SpiderIdentity",
   "Generic",
-  "LGPL",
   "Link between spider and outside elements",
-  VERSION,
-  "Benjamin Otte <in7y118@public.uni-hamburg.de>",
-  "(C) 2002",
-};
+  "Benjamin Otte <in7y118@public.uni-hamburg.de>"
+);
 
 
 /* generic templates 
  * delete me when meging with spider.c
  */
-GST_PAD_TEMPLATE_FACTORY (spider_src_factory,
+static GstStaticPadTemplate spider_src_factory =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
-  GST_PAD_REQUEST,
-  NULL      /* no caps */
+  GST_PAD_ALWAYS,
+  GST_STATIC_CAPS_ANY
 );
 
-GST_PAD_TEMPLATE_FACTORY (spider_sink_factory,
+static GstStaticPadTemplate spider_sink_factory =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
-  GST_PAD_REQUEST,
-  NULL      /* no caps */
+  GST_PAD_ALWAYS,
+  GST_STATIC_CAPS_ANY
 );
 
 /* SpiderIdentity signals and args */
@@ -65,7 +64,7 @@ enum {
 };
 
 enum {
-  ARG_0,
+  ARG_0
   /* FILL ME */
 };
 
@@ -76,8 +75,8 @@ static void			gst_spider_identity_init		(GstSpiderIdentity *spider_identity);
 /* functions set in pads, elements and stuff */
 static void			gst_spider_identity_chain		(GstPad *pad, GstBuffer *buf);
 static GstElementStateReturn	gst_spider_identity_change_state	(GstElement *element);
-static GstPadLinkReturn		gst_spider_identity_link		(GstPad *pad, GstCaps *caps);
-static GstCaps *		gst_spider_identity_getcaps		(GstPad *pad, GstCaps *caps);
+static GstPadLinkReturn		gst_spider_identity_link		(GstPad *pad, const GstCaps *caps);
+static GstCaps *		gst_spider_identity_getcaps		(GstPad *pad);
 /* loop functions */
 static void			gst_spider_identity_dumb_loop		(GstSpiderIdentity *ident);
 static void                     gst_spider_identity_src_loop		(GstSpiderIdentity *ident);
@@ -108,7 +107,10 @@ gst_spider_identity_get_type (void)
       0,
       (GInstanceInitFunc)gst_spider_identity_init,
     };
-    spider_identity_type = g_type_register_static (GST_TYPE_ELEMENT, "GstSpiderIdentity", &spider_identity_info, 0);
+    spider_identity_type = g_type_register_static (GST_TYPE_ELEMENT, "GstSpiderIdentity", 
+						   &spider_identity_info, 0);
+    GST_DEBUG_CATEGORY_INIT (gst_spider_identity_debug, "spideridentity", 
+			     0, "spider autoplugging proxy element");
   }
   return spider_identity_type;
 }
@@ -121,34 +123,28 @@ gst_spider_identity_class_init (GstSpiderIdentityClass *klass)
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
   
   /* add our two pad templates */
-  gst_element_class_add_pad_template (gstelement_class, GST_PAD_TEMPLATE_GET (spider_src_factory));
-  gst_element_class_add_pad_template (gstelement_class, GST_PAD_TEMPLATE_GET (spider_sink_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&spider_src_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&spider_sink_factory));
+  gst_element_class_set_details (gstelement_class, &gst_spider_identity_details);
   
   gstelement_class->change_state = GST_DEBUG_FUNCPTR(gst_spider_identity_change_state);
   gstelement_class->request_new_pad = GST_DEBUG_FUNCPTR(gst_spider_identity_request_new_pad);
 }
 
-static GstBufferPool*
-gst_spider_identity_get_bufferpool (GstPad *pad)
-{
-  GstSpiderIdentity *ident;
-
-  ident = GST_SPIDER_IDENTITY (gst_pad_get_parent (pad));
-
-  return gst_pad_get_bufferpool (ident->src);
-}
-
 static void 
 gst_spider_identity_init (GstSpiderIdentity *ident) 
 {
   /* sink */
-  ident->sink = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (spider_sink_factory), "sink");
+  ident->sink = gst_pad_new_from_template (
+      gst_static_pad_template_get (&spider_sink_factory), "sink");
   gst_element_add_pad (GST_ELEMENT (ident), ident->sink);
   gst_pad_set_link_function (ident->sink, GST_DEBUG_FUNCPTR (gst_spider_identity_link));
   gst_pad_set_getcaps_function (ident->sink, GST_DEBUG_FUNCPTR (gst_spider_identity_getcaps));
-  gst_pad_set_bufferpool_function (ident->sink, GST_DEBUG_FUNCPTR (gst_spider_identity_get_bufferpool));
   /* src */
-  ident->src = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (spider_src_factory), "src");
+  ident->src = gst_pad_new_from_template (
+      gst_static_pad_template_get (&spider_src_factory), "src");
   gst_element_add_pad (GST_ELEMENT (ident), ident->src);
   gst_pad_set_link_function (ident->src, GST_DEBUG_FUNCPTR (gst_spider_identity_link));
   gst_pad_set_getcaps_function (ident->src, GST_DEBUG_FUNCPTR (gst_spider_identity_getcaps));
@@ -156,7 +152,6 @@ gst_spider_identity_init (GstSpiderIdentity *ident)
 
   /* variables */
   ident->plugged = FALSE;
-  
 }
 
 static void 
@@ -227,7 +222,7 @@ gst_spider_identity_new_sink (gchar *name)
 
 /* shamelessly stolen from gstqueue.c to get proxy links */
 static GstPadLinkReturn
-gst_spider_identity_link (GstPad *pad, GstCaps *caps)
+gst_spider_identity_link (GstPad *pad, const GstCaps *caps)
 {
   GstSpiderIdentity *spider_identity = GST_SPIDER_IDENTITY (gst_pad_get_parent (pad));
   GstPad *otherpad;
@@ -237,27 +232,39 @@ gst_spider_identity_link (GstPad *pad, GstCaps *caps)
   else
     otherpad = spider_identity->src;
 
-  if (otherpad != NULL)
-    return gst_pad_proxy_link (otherpad, caps);
-  
-  return GST_PAD_LINK_OK;
+  g_return_val_if_fail (otherpad != NULL, GST_PAD_LINK_REFUSED);
+  if (GST_PAD_PEER (otherpad) == NULL)
+    return GST_PAD_LINK_DELAYED;
+
+  return gst_pad_try_set_caps (otherpad, caps);
 }
 
 static GstCaps*
-gst_spider_identity_getcaps (GstPad *pad, GstCaps *caps)
+gst_spider_identity_getcaps (GstPad *pad)
 {
-  GstSpiderIdentity *spider_identity = GST_SPIDER_IDENTITY (gst_pad_get_parent (pad));
+  GstSpiderIdentity *ident = GST_SPIDER_IDENTITY (gst_pad_get_parent (pad));
   GstPad *otherpad;
 
-  if (pad == spider_identity->src) 
-    otherpad = spider_identity->sink;
+  if (pad == ident->src) 
+    otherpad = ident->sink;
   else
-    otherpad = spider_identity->src;
+    otherpad = ident->src;
 
-  if (otherpad != NULL)
-    return gst_pad_get_allowed_caps (otherpad);
-  
-  return NULL;
+  if (otherpad != NULL) {
+    if (GST_PAD_PEER (otherpad)) {
+      GstCaps *ret = gst_pad_get_allowed_caps (otherpad);
+      if (ident->caps) {
+	GstCaps *ret2 = gst_caps_intersect (ident->caps, ret);
+	gst_caps_free (ret);
+	ret = ret2;
+      }
+      return ret;
+    }
+  }
+  if (ident->caps)
+    return gst_caps_copy (ident->caps);
+
+  return gst_caps_new_any ();
 }
 
 GstPad*
@@ -282,7 +289,6 @@ gst_spider_identity_request_new_pad  (GstElement *element, GstPadTemplate *templ
       gst_element_add_pad (GST_ELEMENT (ident), ident->sink);
       gst_pad_set_link_function (ident->sink, GST_DEBUG_FUNCPTR (gst_spider_identity_link));
       gst_pad_set_getcaps_function (ident->sink, GST_DEBUG_FUNCPTR (gst_spider_identity_getcaps));
-      gst_pad_set_bufferpool_function (ident->sink, GST_DEBUG_FUNCPTR (gst_spider_identity_get_bufferpool));
       return ident->sink;
     case GST_PAD_SRC:
       /* src */
@@ -320,6 +326,9 @@ gst_spider_identity_change_state (GstElement *element)
   g_return_val_if_fail (GST_IS_SPIDER_IDENTITY (ident), GST_STATE_FAILURE);
   
   switch (GST_STATE_TRANSITION (element)) {
+    case GST_STATE_PAUSED_TO_READY:
+      gst_caps_replace (&ident->caps, NULL);
+      break;
     case GST_STATE_PAUSED_TO_PLAYING:
       /* autoplugger check */
       spider = GST_SPIDER (GST_ELEMENT_PARENT (ident));
@@ -329,13 +338,16 @@ gst_spider_identity_change_state (GstElement *element)
       /* start typefinding or plugging */
       if ((GST_RPAD_PEER (ident->sink) != NULL) && (GST_RPAD_PEER (ident->src) == NULL))
       {
-        if (gst_pad_get_caps ((GstPad *) GST_PAD_PEER (ident->sink)) == NULL)
+	GstCaps *caps = gst_pad_get_caps ((GstPad *) GST_PAD_PEER (ident->sink));
+        if (gst_caps_is_any (caps) || gst_caps_is_empty (caps))
         {
           gst_spider_identity_start_type_finding (ident);
+	  gst_caps_free (caps);
           break;
         } else {
           gst_spider_identity_plug (ident);
         }
+	gst_caps_free (caps);
       }
       /* autoplug on src */
       if ((GST_RPAD_PEER (ident->src) != NULL) && (GST_RPAD_PEER (ident->sink) == NULL))
@@ -410,82 +422,112 @@ gst_spider_identity_src_loop (GstSpiderIdentity *ident)
 }
 /* This loop function is only needed when typefinding.
  */
+typedef struct {
+  GstBuffer *buffer;
+  guint best_probability;
+  GstCaps *caps;
+} SpiderTypeFind;
+guint8 *
+spider_find_peek (gpointer data, gint64 offset, guint size)
+{
+  SpiderTypeFind *find = (SpiderTypeFind *) data;
+  gint64 buffer_offset = GST_BUFFER_OFFSET_IS_VALID (find->buffer) ? 
+			 GST_BUFFER_OFFSET (find->buffer) : 0;
+  
+  if (offset >= buffer_offset && offset + size <= buffer_offset + GST_BUFFER_SIZE (find->buffer)) {
+    GST_LOG ("peek %"G_GINT64_FORMAT", %u successful", offset, size);
+    return GST_BUFFER_DATA (find->buffer) + offset - buffer_offset;
+  } else {
+    GST_LOG ("peek %"G_GINT64_FORMAT", %u failed", offset, size);
+    return NULL;
+  }
+}
+static void
+spider_find_suggest (gpointer data, guint probability, const GstCaps *caps)
+{
+  SpiderTypeFind *find = (SpiderTypeFind *) data;
+
+  GST_INFO ("suggest %u, %" GST_PTR_FORMAT, probability, caps);
+  if (probability > find->best_probability) {
+    gst_caps_replace (&find->caps, gst_caps_copy (caps));
+    find->best_probability = probability;
+  }
+}
 static void
 gst_spider_identity_sink_loop_type_finding (GstSpiderIdentity *ident)
 {
-  GstBuffer *buf = NULL;
-  GList *type_list;
-  GstCaps *caps;
-  GstByteStream *bs;
+  GstData *data;
+  GstTypeFind gst_find;
+  SpiderTypeFind find;
+  GList *walk, *type_list = NULL;
 
   g_return_if_fail (GST_IS_SPIDER_IDENTITY (ident));
 
-  /* get a bytestream object */
-  bs = gst_bytestream_new (ident->sink);
-  if (gst_bytestream_peek (bs, &buf, 1) != 1 || !buf) {
-    buf = NULL;
-    g_warning ("Failed to read fake buffer - serious idiocy going on here");
-    goto end;
-  } else {
-    gst_buffer_unref (buf);
-    buf = NULL;
+  data = gst_pad_pull (ident->sink);
+  if (!GST_IS_BUFFER (data)) {
+    gst_spider_identity_chain (ident->sink, GST_BUFFER (data));
+    return;
   }
-
+  
+  find.buffer = GST_BUFFER (data);
   /* maybe there are already valid caps now? */
-  if ((caps = gst_pad_get_caps (ident->sink)) != NULL) {
+  find.caps = gst_pad_get_allowed_caps (ident->sink);
+  if (!gst_caps_is_empty (find.caps) && !gst_caps_is_any (find.caps)) {
     goto plug;
+  } else {
+    gst_caps_free (find.caps);
+    find.caps = NULL;
   }
   
   /* now do the actual typefinding with the supplied buffer */
-  type_list = (GList *) gst_type_get_list ();
+  walk = type_list = gst_type_find_factory_get_list ();
     
-  while (type_list) {
-    GstType *type = (GstType *) type_list->data;
-    GSList *factories = type->factories;
+  find.best_probability = 0;
+  find.caps = NULL;
+  gst_find.data = &find;
+  gst_find.peek = spider_find_peek;
+  gst_find.suggest = spider_find_suggest;
+  while (walk) {
+    GstTypeFindFactory *factory = GST_TYPE_FIND_FACTORY (walk->data);
 
-    while (factories) {
-      GstTypeFactory *factory = GST_TYPE_FACTORY (factories->data);
-      GstTypeFindFunc typefindfunc = (GstTypeFindFunc)factory->typefindfunc;
-
-      GST_DEBUG ("trying typefind function %s", GST_PLUGIN_FEATURE_NAME (factory));
-      if (typefindfunc && (caps = typefindfunc (bs, factory))) {
-        GST_INFO ("typefind function %s found caps", GST_PLUGIN_FEATURE_NAME (factory));
-        if (gst_pad_try_set_caps (ident->src, caps) <= 0) {
-          g_warning ("typefind: found type but peer didn't accept it");
-	  gst_caps_sink (caps);
-        } else {
-          goto plug;
-	}
-      }
-      factories = g_slist_next (factories);
-    }
-    type_list = g_list_next (type_list);
+    GST_DEBUG ("trying typefind function %s", GST_PLUGIN_FEATURE_NAME (factory));
+    gst_type_find_factory_call_function (factory, &gst_find);
+    if (find.best_probability >= GST_TYPE_FIND_MAXIMUM)
+      goto plug;
+    walk = g_list_next (walk);
   }
-  gst_element_error(GST_ELEMENT(ident), "Could not find media type", NULL);
-  buf = GST_BUFFER (gst_event_new (GST_EVENT_EOS));
+  if (find.best_probability > 0)
+    goto plug;
+  GST_ELEMENT_ERROR (ident, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
+  find.buffer = GST_BUFFER (gst_event_new (GST_EVENT_EOS));
 
 end:
-
   /* remove loop function */
   gst_element_set_loop_function (GST_ELEMENT (ident), 
                                 (GstElementLoopFunction) GST_DEBUG_FUNCPTR (gst_spider_identity_dumb_loop));
   
   /* push the buffer */
-  gst_spider_identity_chain (ident->sink, buf);
-
-  /* bytestream no longer needed */
-  gst_bytestream_destroy (bs);
+  gst_spider_identity_chain (ident->sink, find.buffer);
   
   return;
 
 plug:
-  gst_caps_debug (caps, "spider starting caps");
-  gst_caps_sink (caps);
+  GST_INFO ("typefind function found caps"); 
+  ident->caps = find.caps;
+  if (GST_PAD_IS_LINKED (ident->src)) {
+    GstPadLinkReturn ret;
+
+    ret = gst_pad_try_set_caps (ident->src, find.caps);
+    if (GST_PAD_LINK_FAILED (ret)){
+      g_critical("could not set caps on spideridentity src pad\n");
+    }
+  }
+  GST_LOG_OBJECT (ident, "spider starting caps: %" GST_PTR_FORMAT, find.caps);
+  if (type_list)
+    g_list_free (type_list);
 
   gst_spider_identity_plug (ident);
 
-  gst_bytestream_read (bs, &buf, bs->listavail);
-
   goto end;
 }
 
