@@ -26,6 +26,7 @@
 #endif
 
 #include "gstfakesink.h"
+#include <gst/gstmarshal.h>
 
 GST_DEBUG_CATEGORY_STATIC (gst_fakesink_debug);
 #define GST_CAT_DEFAULT gst_fakesink_debug
@@ -56,11 +57,11 @@ enum {
   ARG_LAST_MESSAGE,
 };
 
-GST_PAD_TEMPLATE_FACTORY (fakesink_sink_factory,
+GstStaticPadTemplate fakesink_sink_template = GST_STATIC_PAD_TEMPLATE (
   "sink%d",
   GST_PAD_SINK,
   GST_PAD_REQUEST,
-  GST_CAPS_ANY
+  GST_STATIC_CAPS_ANY
 );
 
 #define GST_TYPE_FAKESINK_STATE_ERROR (gst_fakesink_state_error_get_type())
@@ -84,9 +85,10 @@ gst_fakesink_state_error_get_type (void)
   return fakesink_state_error_type;
 }
 
-static void	gst_fakesink_base_init		(gpointer g_class);
-static void	gst_fakesink_class_init		(GstFakeSinkClass *klass);
-static void	gst_fakesink_init		(GstFakeSink *fakesink);
+#define _do_init(bla) \
+    GST_DEBUG_CATEGORY_INIT (gst_fakesink_debug, "fakesink", 0, "fakesink element");
+
+GST_BOILERPLATE_FULL (GstFakeSink, gst_fakesink, GstElement, GST_TYPE_ELEMENT, _do_init);
 
 static void 	gst_fakesink_set_clock 		(GstElement *element, GstClock *clock);
 static GstPad* 	gst_fakesink_request_new_pad 	(GstElement *element, GstPadTemplate *templ, const
@@ -102,41 +104,18 @@ static GstElementStateReturn
 
 static void	gst_fakesink_chain		(GstPad *pad, GstData *_data);
 
-static GstElementClass *parent_class = NULL;
 static guint gst_fakesink_signals[LAST_SIGNAL] = { 0 };
 
-GType
-gst_fakesink_get_type (void) 
-{
-  static GType fakesink_type = 0;
-
-  if (!fakesink_type) {
-    static const GTypeInfo fakesink_info = {
-      sizeof(GstFakeSinkClass),
-      gst_fakesink_base_init,
-      NULL,
-      (GClassInitFunc)gst_fakesink_class_init,
-      NULL,
-      NULL,
-      sizeof(GstFakeSink),
-      0,
-      (GInstanceInitFunc)gst_fakesink_init,
-    };
-    fakesink_type = g_type_register_static (GST_TYPE_ELEMENT, "GstFakeSink", &fakesink_info, 0);
-  
-    GST_DEBUG_CATEGORY_INIT (gst_fakesink_debug, "fakesink", 0, "fakesink element");
-  }
-  return fakesink_type;
-}
-
 static void
 gst_fakesink_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class); 
   
   gst_element_class_set_details (gstelement_class, &gst_fakesink_details);
-  gst_element_class_add_pad_template (gstelement_class, GST_PAD_TEMPLATE_GET (fakesink_sink_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&fakesink_sink_template));
 }
+
 static void
 gst_fakesink_class_init (GstFakeSinkClass *klass) 
 {
@@ -146,7 +125,6 @@ gst_fakesink_class_init (GstFakeSinkClass *klass)
   gobject_class = (GObjectClass*)klass;
   gstelement_class = (GstElementClass*)klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_NUM_SINKS,
     g_param_spec_int ("num_sinks", "Number of sinks", "The number of sinkpads",
@@ -335,7 +313,7 @@ gst_fakesink_chain (GstPad *pad, GstData *_data)
       case GST_EVENT_DISCONTINUOUS:
         if (fakesink->sync && fakesink->clock) { 
           gint64 value = GST_EVENT_DISCONT_OFFSET (event, 0).value;
-          gst_clock_handle_discont (fakesink->clock, value);
+          gst_element_set_time (GST_ELEMENT (fakesink), value);
 	}
       default:
 	gst_pad_event_default (pad, event);
@@ -345,10 +323,7 @@ gst_fakesink_chain (GstPad *pad, GstData *_data)
   }
 
   if (fakesink->sync && fakesink->clock) { 
-    GstClockID id = gst_clock_new_single_shot_id (fakesink->clock, GST_BUFFER_TIMESTAMP (buf));
-
-    gst_element_clock_wait (GST_ELEMENT (fakesink), id, NULL);
-    gst_clock_id_free (id);
+    gst_element_wait (GST_ELEMENT (fakesink), GST_BUFFER_TIMESTAMP (buf));
   }
 
   if (!fakesink->silent) { 
@@ -417,7 +392,7 @@ gst_fakesink_change_state (GstElement *element)
   return GST_STATE_SUCCESS;
 
 error:
-  gst_element_error (element, "failed state change as requested");
+  GST_ELEMENT_ERROR (element, CORE, STATE_CHANGE, (NULL), (NULL));
   return GST_STATE_FAILURE;
 }
 
