@@ -29,6 +29,7 @@
 #endif
 
 #include "gstfakesrc.h"
+#include <gst/gstmarshal.h>
 
 #define DEFAULT_SIZEMIN		0
 #define DEFAULT_SIZEMAX		4096
@@ -73,11 +74,11 @@ enum {
   ARG_LAST_MESSAGE,
 };
 
-GST_PAD_TEMPLATE_FACTORY (fakesrc_src_factory,
+GstStaticPadTemplate fakesrc_src_template = GST_STATIC_PAD_TEMPLATE (
   "src%d",
   GST_PAD_SRC,
   GST_PAD_REQUEST,
-  GST_CAPS_ANY
+  GST_STATIC_CAPS_ANY
 );
 
 #define GST_TYPE_FAKESRC_OUTPUT (gst_fakesrc_output_get_type())
@@ -98,8 +99,6 @@ gst_fakesrc_output_get_type (void)
   };
   if (!fakesrc_output_type) {
     fakesrc_output_type = g_enum_register_static ("GstFakeSrcOutput", fakesrc_output);
-  
-    GST_DEBUG_CATEGORY_INIT (gst_fakesrc_debug, "fakesrc", 0, "fakesrc element");
   }
   return fakesrc_output_type;
 }
@@ -112,7 +111,6 @@ gst_fakesrc_data_get_type (void)
   static GEnumValue fakesrc_data[] = {
     { FAKESRC_DATA_ALLOCATE, 		"1", "Allocate data"},
     { FAKESRC_DATA_SUBBUFFER, 		"2", "Subbuffer data"},
-    { FAKESRC_DATA_BUFFERPOOL, 		"3", "Use the default buffer pool (forces sizetype=2)"},
     {0, NULL, NULL},
   };
   if (!fakesrc_data_type) {
@@ -157,9 +155,10 @@ gst_fakesrc_filltype_get_type (void)
   return fakesrc_filltype_type;
 }
 
-static void		gst_fakesrc_base_init		(gpointer g_class);
-static void		gst_fakesrc_class_init		(GstFakeSrcClass *klass);
-static void		gst_fakesrc_init		(GstFakeSrc *fakesrc);
+#define _do_init(bla) \
+    GST_DEBUG_CATEGORY_INIT (gst_fakesrc_debug, "fakesrc", 0, "fakesrc element");
+
+GST_BOILERPLATE_FULL (GstFakeSrc, gst_fakesrc, GstElement, GST_TYPE_ELEMENT, _do_init);
 
 static GstPad* 		gst_fakesrc_request_new_pad 	(GstElement *element, GstPadTemplate *templ,const gchar *unused);
 static void 		gst_fakesrc_update_functions 	(GstFakeSrc *src);
@@ -173,39 +172,18 @@ static GstElementStateReturn gst_fakesrc_change_state 	(GstElement *element);
 static GstData*	gst_fakesrc_get			(GstPad *pad);
 static void 		gst_fakesrc_loop		(GstElement *element);
 
-static GstElementClass *parent_class = NULL;
 static guint gst_fakesrc_signals[LAST_SIGNAL] = { 0 };
 
-GType
-gst_fakesrc_get_type (void) 
-{
-  static GType fakesrc_type = 0;
-
-  if (!fakesrc_type) {
-    static const GTypeInfo fakesrc_info = {
-      sizeof(GstFakeSrcClass),
-      gst_fakesrc_base_init,
-      NULL,
-      (GClassInitFunc)gst_fakesrc_class_init,
-      NULL,
-      NULL,
-      sizeof(GstFakeSrc),
-      0,
-      (GInstanceInitFunc)gst_fakesrc_init,
-    };
-    fakesrc_type = g_type_register_static (GST_TYPE_ELEMENT, "GstFakeSrc", &fakesrc_info, 0);
-  }
-  return fakesrc_type;
-}
-
 static void
 gst_fakesrc_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class); 
   
   gst_element_class_set_details (gstelement_class, &gst_fakesrc_details);
-  gst_element_class_add_pad_template (gstelement_class, GST_PAD_TEMPLATE_GET (fakesrc_src_factory));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&fakesrc_src_template));
 }
+
 static void
 gst_fakesrc_class_init (GstFakeSrcClass *klass) 
 {
@@ -215,7 +193,6 @@ gst_fakesrc_class_init (GstFakeSrcClass *klass)
   gobject_class = (GObjectClass*)klass;
   gstelement_class = (GstElementClass*)klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_NUM_SOURCES,
     g_param_spec_int ("num-sources", "num-sources", "Number of sources",
@@ -339,16 +316,28 @@ gst_fakesrc_request_new_pad (GstElement *element, GstPadTemplate *templ, const g
   return srcpad;
 }
 
-GST_PAD_FORMATS_FUNCTION (gst_fakesrc_get_formats,
-  GST_FORMAT_DEFAULT
-)
+static const GstFormat *
+gst_fakesrc_get_formats (GstPad *pad)
+{
+  static const GstFormat formats[] = {
+    GST_FORMAT_DEFAULT,
+    0,
+  };
+  return formats;
+}
 
-GST_PAD_QUERY_TYPE_FUNCTION (gst_fakesrc_get_query_types,
-  GST_QUERY_TOTAL,
-  GST_QUERY_POSITION,
-  GST_QUERY_START,
-  GST_QUERY_SEGMENT_END
-) 
+static const GstQueryType *
+gst_fakesrc_get_query_types (GstPad *pad)
+{
+  static const GstQueryType types[] = {
+    GST_QUERY_TOTAL,
+    GST_QUERY_POSITION,
+    GST_QUERY_START,
+    GST_QUERY_SEGMENT_END,
+    0,
+  };
+  return types;
+}
 
 static gboolean
 gst_fakesrc_query (GstPad *pad, GstQueryType type,
@@ -375,11 +364,17 @@ gst_fakesrc_query (GstPad *pad, GstQueryType type,
   return TRUE;
 }
 
-GST_PAD_EVENT_MASK_FUNCTION (gst_fakesrc_get_event_mask,
-  { GST_EVENT_SEEK, GST_SEEK_FLAG_FLUSH },
-  { GST_EVENT_SEEK_SEGMENT, GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_SEGMENT_LOOP },
-  { GST_EVENT_FLUSH, 0 }
-)
+static const GstEventMask *
+gst_fakesrc_get_event_mask (GstPad *pad)
+{
+  static const GstEventMask masks[] = {
+    { GST_EVENT_SEEK, GST_SEEK_FLAG_FLUSH },
+    { GST_EVENT_SEEK_SEGMENT, GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_SEGMENT_LOOP },
+    { GST_EVENT_FLUSH, 0 },
+    { 0, 0 },
+  };
+  return masks;
+}
 
 static gboolean
 gst_fakesrc_event_handler (GstPad *pad, GstEvent *event)
@@ -486,19 +481,6 @@ gst_fakesrc_set_property (GObject *object, guint prop_id, const GValue *value, G
           src->parent = NULL;
         }
       }
-      
-      if (src->data == FAKESRC_DATA_BUFFERPOOL) {
-        if (src->sizetype != FAKESRC_SIZETYPE_FIXED)
-          g_object_set (src, "sizetype", FAKESRC_SIZETYPE_FIXED, NULL);
-        
-        if (!src->pool) 
-          src->pool = gst_buffer_pool_get_default (src->sizemax, 10);
-      } else {
-        if (src->pool) {
-          gst_buffer_pool_unref (src->pool);
-          src->pool = NULL;
-        }
-      }
       break;
     case ARG_SIZETYPE:
       src->sizetype = g_value_get_enum (value);
@@ -726,10 +708,6 @@ gst_fakesrc_create_buffer (GstFakeSrc *src)
       }
       gst_fakesrc_prepare_buffer (src, buf);
       break;
-    case FAKESRC_DATA_BUFFERPOOL:
-      buf = gst_buffer_new_from_pool (src->pool, 0, 0);
-      gst_fakesrc_prepare_buffer (src, buf);
-      break;
     default:
       g_warning ("fakesrc: dunno how to allocate buffers !");
       buf = gst_buffer_new();
@@ -867,10 +845,6 @@ gst_fakesrc_change_state (GstElement *element)
         gst_buffer_unref (fakesrc->parent);
         fakesrc->parent = NULL;
       }
-      if (fakesrc->pool) {
-        gst_buffer_pool_unref (fakesrc->pool);
-        fakesrc->pool = NULL;
-      }
       g_free (fakesrc->last_message);
       fakesrc->last_message = NULL;
       break;
