@@ -28,8 +28,6 @@
 
 G_BEGIN_DECLS
 
-extern GstElementDetails gst_fakesrc_details;
-GST_DEBUG_CATEGORY_EXTERN(gst_fakesrc_debug);
 
 typedef enum {
   FAKESRC_FIRST_LAST_LOOP = 1,
@@ -45,7 +43,6 @@ typedef enum {
 typedef enum {
   FAKESRC_DATA_ALLOCATE = 1,
   FAKESRC_DATA_SUBBUFFER,
-  FAKESRC_DATA_BUFFERPOOL
 } GstFakeSrcDataType;
 
 typedef enum {
@@ -105,7 +102,6 @@ struct _GstFakeSrc {
   gboolean 	 signal_handoffs;
   gboolean 	 dump;
   gboolean 	 need_flush;
-  GstBufferPool *pool;
 
   gchar		*last_message;
 };
