@@ -35,6 +35,8 @@
 #include <errno.h>
 #include <string.h>
 
+#include "../gst-i18n-lib.h"
+
 
 /**********************************************************************
  * GStreamer Default File Source
@@ -97,30 +99,45 @@ enum {
   ARG_FD,
   ARG_BLOCKSIZE,
   ARG_MMAPSIZE,
-  ARG_TOUCH,
+  ARG_TOUCH
 };
 
-GST_PAD_EVENT_MASK_FUNCTION (gst_filesrc_get_event_mask,
-  { GST_EVENT_SEEK, GST_SEEK_METHOD_CUR | 
-	  	    GST_SEEK_METHOD_SET | 
-		    GST_SEEK_METHOD_END | 
-		    GST_SEEK_FLAG_FLUSH },
-  { GST_EVENT_FLUSH, 0 },
-  { GST_EVENT_SIZE, 0 }
-)
+static const GstEventMask *
+gst_filesrc_get_event_mask (GstPad *pad)
+{
+  static const GstEventMask masks[] = {
+    { GST_EVENT_SEEK, GST_SEEK_METHOD_CUR | 
+                      GST_SEEK_METHOD_SET | 
+                      GST_SEEK_METHOD_END | 
+                      GST_SEEK_FLAG_FLUSH },
+    { GST_EVENT_FLUSH, 0 },
+    { GST_EVENT_SIZE, 0 },
+    { 0, 0 }
+  };
+  return masks;
+}
 
-GST_PAD_QUERY_TYPE_FUNCTION (gst_filesrc_get_query_types,
-  GST_QUERY_TOTAL,
-  GST_QUERY_POSITION
-)
+static const GstQueryType *
+gst_filesrc_get_query_types (GstPad *pad)
+{
+  static const GstQueryType types[] = {
+    GST_QUERY_TOTAL,
+    GST_QUERY_POSITION,
+    0
+  };
+  return types;
+}
 
-GST_PAD_FORMATS_FUNCTION (gst_filesrc_get_formats,
-  GST_FORMAT_BYTES
-)
+static const GstFormat *
+gst_filesrc_get_formats (GstPad *pad)
+{
+  static const GstFormat formats[] = {
+    GST_FORMAT_BYTES,
+    0,
+  };
+  return formats;
+}
 
-static void		gst_filesrc_base_init		(gpointer g_class);
-static void		gst_filesrc_class_init		(GstFileSrcClass *klass);
-static void		gst_filesrc_init		(GstFileSrc *filesrc);
 static void 		gst_filesrc_dispose 		(GObject *object);
 
 static void		gst_filesrc_set_property	(GObject *object, guint prop_id, 
@@ -138,41 +155,20 @@ static GstElementStateReturn	gst_filesrc_change_state	(GstElement *element);
 
 static void		gst_filesrc_uri_handler_init	(gpointer g_iface, gpointer iface_data);
 
-
-static GstElementClass *parent_class = NULL;
-/*static guint gst_filesrc_signals[LAST_SIGNAL] = { 0 };*/
-
-GType
-gst_filesrc_get_type(void)
+static void
+_do_init (GType filesrc_type)
 {
-  static GType filesrc_type = 0;
-
-  if (!filesrc_type) {
-    static const GTypeInfo filesrc_info = {
-      sizeof(GstFileSrcClass), 
-      gst_filesrc_base_init,
-      NULL,
-      (GClassInitFunc)gst_filesrc_class_init,
-      NULL,
-      NULL,
-      sizeof(GstFileSrc),
-      0,
-      (GInstanceInitFunc)gst_filesrc_init,
-    };
-    static const GInterfaceInfo urihandler_info = {
-      gst_filesrc_uri_handler_init,
-      NULL,
-      NULL
-    };
-    filesrc_type = g_type_register_static (GST_TYPE_ELEMENT, "GstFileSrc", &filesrc_info, 0);
-    
-    g_type_add_interface_static (filesrc_type, GST_TYPE_URI_HANDLER, &urihandler_info);
-    
-    GST_DEBUG_CATEGORY_INIT (gst_filesrc_debug, "filesrc", 0, "filesrc element");
-  }
-  return filesrc_type;
+  static const GInterfaceInfo urihandler_info = {
+    gst_filesrc_uri_handler_init,
+    NULL,
+    NULL
+  };
+  g_type_add_interface_static (filesrc_type, GST_TYPE_URI_HANDLER, &urihandler_info);
+  GST_DEBUG_CATEGORY_INIT (gst_filesrc_debug, "filesrc", 0, "filesrc element");
 }
 
+GST_BOILERPLATE_FULL (GstFileSrc, gst_filesrc, GstElement, GST_TYPE_ELEMENT, _do_init);
+    
 static void
 gst_filesrc_base_init (gpointer g_class)
 {
@@ -188,7 +184,6 @@ gst_filesrc_class_init (GstFileSrcClass *klass)
 
   gobject_class = (GObjectClass*)klass;
 
-  parent_class = g_type_class_peek_parent (klass);
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FD,
     g_param_spec_int ("fd", "File-descriptor", "File-descriptor for the file being mmap()d",
@@ -215,19 +210,6 @@ gst_filesrc_class_init (GstFileSrcClass *klass)
   gstelement_class->change_state = gst_filesrc_change_state;
 }
 
-static gint
-gst_filesrc_bufcmp (gconstpointer a, gconstpointer b)
-{
-/*  GstBuffer *bufa = (GstBuffer *)a, *bufb = (GstBuffer *)b;*/
-
-  /* sort first by offset, then in reverse by size */
-  if (GST_BUFFER_OFFSET(a) < GST_BUFFER_OFFSET(b)) return -1;
-  else if (GST_BUFFER_OFFSET(a) > GST_BUFFER_OFFSET(b)) return 1;
-  else if (GST_BUFFER_SIZE(a) > GST_BUFFER_SIZE(b)) return -1;
-  else if (GST_BUFFER_SIZE(a) < GST_BUFFER_SIZE(b)) return 1;
-  else return 0;
-}
-
 static void
 gst_filesrc_init (GstFileSrc *src)
 {
@@ -253,9 +235,6 @@ gst_filesrc_init (GstFileSrc *src)
   src->mapbuf = NULL;
   src->mapsize = DEFAULT_MMAPSIZE;		/* default is 4MB */
 
-  src->map_regions = g_tree_new (gst_filesrc_bufcmp);
-  src->map_regions_lock = g_mutex_new();
-
   src->seek_happened = FALSE;
 }
 
@@ -268,8 +247,6 @@ gst_filesrc_dispose (GObject *object)
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 
-  g_tree_destroy (src->map_regions);
-  g_mutex_free (src->map_regions_lock);
   if (src->filename)
     g_free (src->filename);
   if (src->uri)
@@ -280,7 +257,8 @@ static gboolean
 gst_filesrc_set_location (GstFileSrc *src, const gchar *location)
 {
   /* the element must be stopped in order to do this */
-  if (GST_STATE (src) == GST_STATE_PLAYING)
+  if (GST_STATE (src) != GST_STATE_READY &&
+      GST_STATE (src) != GST_STATE_NULL)
     return FALSE;
 
   if (src->filename) g_free (src->filename);
@@ -322,7 +300,7 @@ gst_filesrc_set_property (GObject *object, guint prop_id, const GValue *value, G
         src->mapsize = g_value_get_ulong (value);
         g_object_notify (G_OBJECT (src), "mmapsize");
       } else {
-        GST_INFO_OBJECT (src, "invalid mapsize, must a multiple of pagesize, which is %d", 
+        GST_INFO_OBJECT (src, "invalid mapsize, must be a multiple of pagesize, which is %d", 
 	          src->pagesize);
       }
       break;
@@ -370,19 +348,8 @@ gst_filesrc_get_property (GObject *object, guint prop_id, GValue *value, GParamS
 static void
 gst_filesrc_free_parent_mmap (GstBuffer *buf)
 {
-  GstFileSrc *src = GST_FILESRC (GST_BUFFER_POOL_PRIVATE (buf));
-
-  GST_LOG_OBJECT (src, "freeing mmap()d buffer at %"G_GUINT64_FORMAT"+%u", 
-		  GST_BUFFER_OFFSET (buf), GST_BUFFER_SIZE (buf));
-
-  /* remove the buffer from the list of available mmap'd regions */
-  g_mutex_lock (src->map_regions_lock);
-  g_tree_remove (src->map_regions, buf);
-  /* check to see if the tree is empty */
-  if (g_tree_nnodes (src->map_regions) == 0) {
-    /* we have to free the bufferpool we don't have yet */
-  }
-  g_mutex_unlock (src->map_regions_lock);
+  GST_LOG ("freeing mmap()d buffer at %"G_GUINT64_FORMAT"+%u", 
+      GST_BUFFER_OFFSET (buf), GST_BUFFER_SIZE (buf));
 
 #ifdef MADV_DONTNEED
   /* madvise to tell the kernel what to do with it */
@@ -392,15 +359,12 @@ gst_filesrc_free_parent_mmap (GstBuffer *buf)
   munmap (GST_BUFFER_DATA (buf), GST_BUFFER_MAXSIZE (buf));
   /* cast to unsigned long, since there's no gportable way to print
    * guint64 as hex */
-  GST_LOG_OBJECT (src, "unmapped region %08lx+%08lx at %p", 
-		  (unsigned long) GST_BUFFER_OFFSET (buf),
-		  (unsigned long) GST_BUFFER_MAXSIZE (buf), 
-		  GST_BUFFER_DATA (buf));
+  GST_LOG ("unmapped region %08lx+%08lx at %p", 
+      (unsigned long) GST_BUFFER_OFFSET (buf),
+      (unsigned long) GST_BUFFER_MAXSIZE (buf), 
+      GST_BUFFER_DATA (buf));
 
   GST_BUFFER_DATA (buf) = NULL;
-
-  g_object_unref (src);
-  gst_buffer_default_free (buf);
 }
 
 static GstBuffer *
@@ -416,7 +380,7 @@ gst_filesrc_map_region (GstFileSrc *src, off_t offset, size_t size)
   mmapregion = mmap (NULL, size, PROT_READ, MAP_SHARED, src->fd, offset);
 
   if (mmapregion == NULL) {
-    gst_element_error (GST_ELEMENT (src), "couldn't map file");
+    GST_ELEMENT_ERROR (src, RESOURCE, TOO_LAZY, (NULL), ("mmap call failed."));
     return NULL;
   }
   else if (mmapregion == MAP_FAILED) {
@@ -442,14 +406,10 @@ gst_filesrc_map_region (GstFileSrc *src, off_t offset, size_t size)
   GST_BUFFER_SIZE (buf) = size;
   GST_BUFFER_MAXSIZE (buf) = size;
   GST_BUFFER_OFFSET (buf) = offset;
+  GST_BUFFER_OFFSET_END (buf) = offset + size;
   GST_BUFFER_TIMESTAMP (buf) = GST_CLOCK_TIME_NONE;
-  GST_BUFFER_POOL_PRIVATE (buf) = src;
-  g_object_ref (src);
-  GST_BUFFER_FREE_FUNC (buf) = (GstDataFreeFunction) gst_filesrc_free_parent_mmap;
-
-  g_mutex_lock (src->map_regions_lock);
-  g_tree_insert (src->map_regions,buf,buf);
-  g_mutex_unlock (src->map_regions_lock);
+  GST_BUFFER_PRIVATE (buf) = src;
+  GST_BUFFER_FREE_DATA_FUNC (buf) = gst_filesrc_free_parent_mmap;
 
   return buf;
 }
@@ -485,25 +445,6 @@ gst_filesrc_map_small_region (GstFileSrc *src, off_t offset, size_t size)
   return gst_filesrc_map_region(src,offset,size);
 }
 
-typedef struct {
-  off_t offset;
-  off_t size;
-} GstFileSrcRegion;
-
-/* This allows us to search for a potential mmap region. */
-static gint
-gst_filesrc_search_region_match (gpointer a, gpointer b)
-{
-  GstFileSrcRegion *r = (GstFileSrcRegion *)b;
-
-  /* trying to walk b down the tree, current node is a */
-  if (r->offset < GST_BUFFER_OFFSET(a)) return -1;
-  else if (r->offset >= (GST_BUFFER_OFFSET(a) + GST_BUFFER_SIZE(a))) return 1;
-  else if ((r->offset + r->size) <= (GST_BUFFER_OFFSET(a) + GST_BUFFER_SIZE(a))) return 0;
-
-  return -2;
-}
-
 /**
  * gst_filesrc_get_mmap:
  * @pad: #GstPad to push a buffer from
@@ -513,10 +454,9 @@ gst_filesrc_search_region_match (gpointer a, gpointer b)
 static GstBuffer *
 gst_filesrc_get_mmap (GstFileSrc *src)
 {
-  GstBuffer *buf = NULL, *map;
+  GstBuffer *buf = NULL;
   size_t readsize, mapsize;
   off_t readend,mapstart,mapend;
-  GstFileSrcRegion region;
   int i;
 
   /* calculate end pointers so we don't have to do so repeatedly later */
@@ -577,54 +517,39 @@ gst_filesrc_get_mmap (GstFileSrc *src)
   if (buf == NULL) {
     /* first check to see if there's a map that covers the right region already */
     GST_LOG_OBJECT (src, "searching for mapbuf to cover %llu+%d",src->curoffset,readsize);
-    region.offset = src->curoffset;
-    region.size = readsize;
-    map = g_tree_search (src->map_regions,
-			 (GCompareFunc) gst_filesrc_search_region_match,
-			 (gpointer)&region);
+    
+    /* if the read buffer crosses a mmap region boundary, create a one-off region */
+    if ((src->curoffset / src->mapsize) != (readend / src->mapsize)) {
+      GST_LOG_OBJECT (src, "read buf %llu+%d crosses a %d-byte boundary, creating a one-off",
+	     src->curoffset,readsize,src->mapsize);
+      buf = gst_filesrc_map_small_region (src, src->curoffset, readsize);
+      if (buf == NULL)
+	return NULL;
 
-    /* if we found an exact match, subbuffer it */
-    if (map != NULL) {
-      GST_LOG_OBJECT (src, "found mapbuf at %"G_GUINT64_FORMAT"+%u, creating subbuffer",
-		      GST_BUFFER_OFFSET (map), GST_BUFFER_SIZE (map));
-      buf = gst_buffer_create_sub (map, src->curoffset - GST_BUFFER_OFFSET(map), readsize);
-      GST_BUFFER_OFFSET (buf) = src->curoffset;
-
-    /* otherwise we need to create something out of thin air */
+    /* otherwise we will create a new mmap region and set it to the default */
     } else {
-      /* if the read buffer crosses a mmap region boundary, create a one-off region */
-      if ((src->curoffset / src->mapsize) != (readend / src->mapsize)) {
-        GST_LOG_OBJECT (src, "read buf %llu+%d crosses a %d-byte boundary, creating a one-off",
-               src->curoffset,readsize,src->mapsize);
-        buf = gst_filesrc_map_small_region (src, src->curoffset, readsize);
-	if (buf == NULL)
-          return NULL;
+      size_t mapsize;
 
-      /* otherwise we will create a new mmap region and set it to the default */
-      } else {
-	size_t mapsize;
+      off_t nextmap = src->curoffset - (src->curoffset % src->mapsize);
+      GST_LOG_OBJECT (src, "read buf %llu+%d in new mapbuf at %llu+%d, mapping and subbuffering",
+	     src->curoffset, readsize, nextmap, src->mapsize);
+      /* first, we're done with the old mapbuf */
+      gst_buffer_unref(src->mapbuf);
+      mapsize = src->mapsize;
 
-        off_t nextmap = src->curoffset - (src->curoffset % src->mapsize);
-        GST_LOG_OBJECT (src, "read buf %llu+%d in new mapbuf at %llu+%d, mapping and subbuffering",
-               src->curoffset, readsize, nextmap, src->mapsize);
-        /* first, we're done with the old mapbuf */
-        gst_buffer_unref(src->mapbuf);
-	mapsize = src->mapsize;
-
-	/* double the mapsize as long as the readsize is smaller */
-	while (readsize - (src->curoffset - nextmap) > mapsize) {
-          GST_LOG_OBJECT (src, "readsize smaller then mapsize %08x %d", readsize, mapsize);
-          mapsize <<=1;
-	}
-        /* create a new one */
-        src->mapbuf = gst_filesrc_map_region (src, nextmap, mapsize);
-	if (src->mapbuf == NULL)
-          return NULL;
-
-        /* subbuffer it */
-        buf = gst_buffer_create_sub (src->mapbuf, src->curoffset - nextmap, readsize);
-        GST_BUFFER_OFFSET (buf) = GST_BUFFER_OFFSET (src->mapbuf) + src->curoffset - nextmap;
+      /* double the mapsize as long as the readsize is smaller */
+      while (readsize - (src->curoffset - nextmap) > mapsize) {
+	GST_LOG_OBJECT (src, "readsize smaller then mapsize %08x %d", readsize, mapsize);
+	mapsize <<=1;
       }
+      /* create a new one */
+      src->mapbuf = gst_filesrc_map_region (src, nextmap, mapsize);
+      if (src->mapbuf == NULL)
+	return NULL;
+
+      /* subbuffer it */
+      buf = gst_buffer_create_sub (src->mapbuf, src->curoffset - nextmap, readsize);
+      GST_BUFFER_OFFSET (buf) = GST_BUFFER_OFFSET (src->mapbuf) + src->curoffset - nextmap;
     }
   }
 
@@ -661,15 +586,18 @@ gst_filesrc_get_read (GstFileSrc *src)
 
   ret = read (src->fd, GST_BUFFER_DATA (buf), readsize);
   if (ret < 0){
-    gst_element_error (GST_ELEMENT (src), "reading file (%s)",
-        strerror (errno), NULL);
+    GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL), GST_ERROR_SYSTEM);
     return NULL;
   }
   if (ret < readsize) {
-    gst_element_error (GST_ELEMENT (src), "unexpected end of file", NULL);
+    GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL), ("unexpected end of file."));
     return NULL;
   }
 
+  GST_BUFFER_SIZE (buf) = readsize;
+  GST_BUFFER_MAXSIZE (buf) = readsize;
+  GST_BUFFER_OFFSET (buf) = src->curoffset;
+  GST_BUFFER_OFFSET_END (buf) = src->curoffset + readsize;
   src->curoffset += readsize;
 
   return buf;
@@ -684,6 +612,12 @@ gst_filesrc_get (GstPad *pad)
   src = GST_FILESRC (gst_pad_get_parent (pad));
   g_return_val_if_fail (GST_FLAG_IS_SET (src, GST_FILESRC_OPEN), NULL);
 
+  /* check for flush */
+  if (src->need_flush) {
+    src->need_flush = FALSE;
+    GST_DEBUG_OBJECT (src, "sending flush");
+    return GST_DATA (gst_event_new_flush ());
+  }
   /* check for seek */
   if (src->seek_happened) {
     GstEvent *event;
@@ -691,15 +625,8 @@ gst_filesrc_get (GstPad *pad)
     src->seek_happened = FALSE;
     GST_DEBUG_OBJECT (src, "sending discont");
     event = gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, src->curoffset, NULL);
-    src->need_flush = FALSE;
     return GST_DATA (event);
   }
-  /* check for flush */
-  if (src->need_flush) {
-    src->need_flush = FALSE;
-    GST_DEBUG_OBJECT (src, "sending flush");
-    return GST_DATA (gst_event_new_flush ());
-  }
 
   /* check for EOF */
   g_assert (src->curoffset <= src->filelen);
@@ -736,18 +663,38 @@ gst_filesrc_check_filesize (GstFileSrc *src)
   return TRUE;
 }
 /* open the file and mmap it, necessary to go to READY state */
-static gboolean 
+static gboolean
 gst_filesrc_open_file (GstFileSrc *src)
 {
   g_return_val_if_fail (!GST_FLAG_IS_SET (src ,GST_FILESRC_OPEN), FALSE);
 
+  if (src->filename == NULL)
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND,
+	                 (_("No filename specified.")), (NULL));
+    return FALSE;
+  }
+
+  if (src->filename == NULL)
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND,
+	                 (_("No file specified for reading.")), (NULL));
+    return FALSE;
+  }
+
+
   GST_INFO_OBJECT (src, "opening file %s",src->filename);
 
   /* open the file */
   src->fd = open (src->filename, O_RDONLY);
-  if (src->fd < 0) {
-    gst_element_error (GST_ELEMENT (src), "opening file \"%s\" (%s)", 
-    		       src->filename, strerror (errno), NULL);
+  if (src->fd < 0)
+  {
+    if (errno == ENOENT)
+      GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND, (NULL), (NULL));
+    else
+      GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+	                 (_("Could not open file \"%s\" for reading."), src->filename),
+                         GST_ERROR_SYSTEM);
     return FALSE;
   } else {
     /* check if it is a regular file, otherwise bail out */
@@ -756,8 +703,9 @@ gst_filesrc_open_file (GstFileSrc *src)
     fstat(src->fd, &stat_results);
 
     if (!S_ISREG(stat_results.st_mode)) {
-      gst_element_error (GST_ELEMENT (src), "opening file \"%s\" failed. it isn't a regular file", 
-      					src->filename, NULL);
+      GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+	                   (_("File \"%s\" isn't a regular file."), src->filename),
+                           (NULL));
       close(src->fd);
       return FALSE;
     }
@@ -991,4 +939,3 @@ gst_filesrc_uri_handler_init (gpointer g_iface, gpointer iface_data)
   iface->get_uri = gst_filesrc_uri_get_uri;
   iface->set_uri = gst_filesrc_uri_set_uri;
 }
-
