@@ -24,6 +24,8 @@
 #include <stdio.h>
 
 #include "gst_private.h"
+#include "gst-i18n-lib.h"
+#include <locale.h>		/* for LC_ALL */
 
 #include "gst.h"
 #include "gstqueue.h"
@@ -55,13 +57,16 @@ static gboolean gst_initialized = FALSE;
 static gboolean _gst_initialization_failure = FALSE;
 extern gint _gst_trace_on;
 
+/* set to TRUE when segfaults need to be left as is */
+gboolean _gst_disable_segtrap = FALSE;
+
 extern GThreadFunctions gst_thread_dummy_functions;
 
 
 static void	load_plugin_func	(gpointer data, gpointer user_data);
 static void	init_popt_callback	(poptContext context,
 		                         enum poptCallbackReason reason,
-                                         const struct poptOption *option,
+                                         const GstPoptOption *option,
 					 const char *arg, void *data);
 static gboolean	init_pre		(void);
 static gboolean	init_post		(void);
@@ -85,7 +90,7 @@ debug_log_handler (const gchar *log_domain,
 enum {
   ARG_VERSION=1,
   ARG_FATAL_WARNINGS,
-#ifndef GST_DISABLE_GST_DEBUG 
+#ifndef GST_DISABLE_GST_DEBUG
   ARG_DEBUG_LEVEL,
   ARG_DEBUG,
   ARG_DEBUG_DISABLE,
@@ -96,6 +101,7 @@ enum {
   ARG_PLUGIN_SPEW,
   ARG_PLUGIN_PATH,
   ARG_PLUGIN_LOAD,
+  ARG_SEGTRAP_DISABLE,
   ARG_SCHEDULER,
   ARG_REGISTRY
 };
@@ -107,23 +113,26 @@ enum {
 /* default scheduler, can be changed in gstscheduler.h with
  * the GST_SCHEDULER_DEFAULT_NAME define.
  */
-static const struct poptOption gstreamer_options[] = {
+static const GstPoptOption gstreamer_options[] = {
   {NULL, NUL, POPT_ARG_CALLBACK|POPT_CBFLAG_PRE|POPT_CBFLAG_POST, (void *) &init_popt_callback, 0, NULL, NULL},
+  /* make sure we use our GETTEXT_PACKAGE as the domain for popt translations */
+  {NULL, NUL, POPT_ARG_INTL_DOMAIN, GETTEXT_PACKAGE, 0, NULL, NULL},
   {"gst-version",        NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_VERSION,        N_("Print the GStreamer version"), NULL},
   {"gst-fatal-warnings", NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_FATAL_WARNINGS, N_("Make all warnings fatal"), NULL},
-#ifndef GST_DISABLE_GST_DEBUG 
-  {"gst-debug-level",    NUL, POPT_ARG_INT|POPT_ARGFLAG_STRIP,    NULL, ARG_DEBUG_LEVEL,  N_("default debug level from 1 (only error) to 5 (anything) or 0 for no output"), "LEVEL"},
-  {"gst-debug",          NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_DEBUG,          N_("colon-seperated list of category_name=level pairs to set specific levels for the individual categories.\nExample:GST_AUTOPLUG=5:GST_ELEMENT_*=3"), "CATEGORIES"},
-  {"gst-debug-no-color", NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_DEBUG_NO_COLOR, N_("disable color debugging output"), NULL},
-  {"gst-disable-debug",  NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_DEBUG_DISABLE,  N_("disable debugging")},
-  {"gst-debug-help",     NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_DEBUG_HELP,     N_("print available debug categories and exit"), NULL},
+#ifndef GST_DISABLE_GST_DEBUG
+  {"gst-debug-level",    NUL, POPT_ARG_INT|POPT_ARGFLAG_STRIP,    NULL, ARG_DEBUG_LEVEL,  N_("Default debug level from 1 (only error) to 5 (anything) or 0 for no output"), N_("LEVEL")},
+  {"gst-debug",          NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_DEBUG,          N_("Comma-separated list of category_name:level pairs to set specific levels for the individual categories. Example: GST_AUTOPLUG:5,GST_ELEMENT_*:3"), N_("CATEGORIES")},
+  {"gst-debug-no-color", NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_DEBUG_NO_COLOR, N_("Disable color debugging output"), NULL},
+  {"gst-disable-debug",  NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_DEBUG_DISABLE,  N_("Disable debugging")},
+  {"gst-debug-help",     NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_DEBUG_HELP,     N_("Print available debug categories and exit"), NULL},
 #endif
   {"gst-disable-cpu-opt",NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,	  NULL, ARG_DISABLE_CPU_OPT,N_("Disable accelerated CPU instructions"), NULL},
-  {"gst-plugin-spew",    NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_PLUGIN_SPEW,    N_("enable verbose plugin loading diagnostics"), NULL},
-  {"gst-plugin-path",    NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_PLUGIN_PATH,    N_("'" G_SEARCHPATH_SEPARATOR_S "'--separated path list for loading plugins"), "PATHS"},
-  {"gst-plugin-load",    NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_PLUGIN_LOAD,    N_("comma-separated list of plugins to preload in addition to the list stored in env variable GST_PLUGIN_PATH"), "PLUGINS"},
-  {"gst-scheduler",      NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_SCHEDULER,      N_("scheduler to use ('"GST_SCHEDULER_DEFAULT_NAME"' is the default)"), "SCHEDULER"},
-  {"gst-registry",       NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_REGISTRY,       N_("registry to use") , "REGISTRY"},
+  {"gst-plugin-spew",    NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_PLUGIN_SPEW,    N_("Enable verbose plugin loading diagnostics"), NULL},
+  {"gst-plugin-path",    NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_PLUGIN_PATH,    N_("path list for loading plugins (separated by '" G_SEARCHPATH_SEPARATOR_S "')"),N_("PATHS")},
+  {"gst-plugin-load",    NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_PLUGIN_LOAD,    N_("Comma-separated list of plugins to preload in addition to the list stored in env variable GST_PLUGIN_PATH"), N_("PLUGINS")},
+  {"gst-disable-segtrap",NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_SEGTRAP_DISABLE,N_("Disable trapping of segmentation faults during plugin loading"), NULL},
+  {"gst-scheduler",      NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_SCHEDULER,      N_("Scheduler to use ('"GST_SCHEDULER_DEFAULT_NAME"' is the default)"), N_("SCHEDULER")},
+  {"gst-registry",       NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_REGISTRY,       N_("Registry to use") , N_("REGISTRY")},
   POPT_TABLEEND
 };
 
@@ -132,13 +141,16 @@ static const struct poptOption gstreamer_options[] = {
  *
  * Returns a popt option table with GStreamer's argument specifications. The
  * table is set up to use popt's callback method, so whenever the parsing is
- * actually performed (via a poptGetContext()), the GStreamer libraries will
+ * actually performed (via poptGetContext), the GStreamer libraries will
  * be initialized.
  *
+ * This function is useful if you want to integrate GStreamer with other
+ * libraries that use popt.
+ *
  * Returns: a pointer to the static GStreamer option table.
  * No free is necessary.
  */
-const struct poptOption *
+const GstPoptOption *
 gst_init_get_popt_table (void)
 {
   return gstreamer_options;
@@ -156,7 +168,7 @@ gst_init_get_popt_table (void)
  * for some reason.  If you want your program to fail fatally,
  * use gst_init() instead.
  *
- * Returns: TRUE if GStreamer coul be initialized
+ * Returns: %TRUE if GStreamer could be initialized.
  */
 gboolean
 gst_init_check (int *argc, char **argv[])
@@ -198,7 +210,7 @@ gst_init (int *argc, char **argv[])
  */
 void
 gst_init_with_popt_table (int *argc, char **argv[],
-	                  const struct poptOption *popt_options)
+	                  const GstPoptOption *popt_options)
 {
   if (!gst_init_check_with_popt_table (argc, argv, popt_options)) {
     g_print ("Could not initialize GStreamer !\n");
@@ -215,24 +227,24 @@ gst_init_with_popt_table (int *argc, char **argv[],
  * setting up internal path lists,
  * registering built-in elements, and loading standard plugins.
  *
- * Returns: TRUE if GStreamer coul be initialized
+ * Returns: %TRUE if GStreamer could be initialized.
  */
 gboolean
 gst_init_check_with_popt_table (int *argc, char **argv[],
-		                const struct poptOption *popt_options)
+		                const GstPoptOption *popt_options)
 {
   poptContext context;
   gint nextopt;
-  struct poptOption *options;
-  struct poptOption options_with[] = {
+  GstPoptOption *options;
+  GstPoptOption options_with[] = {
     {NULL, NUL, POPT_ARG_INCLUDE_TABLE, poptHelpOptions, 				 0, "Help options:", NULL},
-    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, (struct poptOption *) gstreamer_options,	 0, "GStreamer options:", NULL},
-    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, (struct poptOption *) popt_options, 		 0, "Application options:", NULL},
+    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, (GstPoptOption *) gstreamer_options,	 0, "GStreamer options:", NULL},
+    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, (GstPoptOption *) popt_options, 		 0, "Application options:", NULL},
     POPT_TABLEEND
   };
-  struct poptOption options_without[] = {
+  GstPoptOption options_without[] = {
     {NULL, NUL, POPT_ARG_INCLUDE_TABLE, poptHelpOptions, 				 0, "Help options:", NULL},
-    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, (struct poptOption *) gstreamer_options,	 0, "GStreamer options:", NULL},
+    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, (GstPoptOption *) gstreamer_options,	 0, "GStreamer options:", NULL},
     POPT_TABLEEND
   };
 
@@ -278,24 +290,24 @@ gst_init_check_with_popt_table (int *argc, char **argv[],
   }
 
   *argc = poptStrippedArgv (context, *argc, *argv);
-  
+
   poptFreeContext (context);
 
   return TRUE;
 }
 
 #ifndef GST_DISABLE_REGISTRY
-static void 
+static void
 add_path_func (gpointer data, gpointer user_data)
 {
   GstRegistry *registry = GST_REGISTRY (user_data);
-  
+
   GST_INFO ("Adding plugin path: \"%s\"", (gchar *) data);
   gst_registry_add_path (registry, (gchar *)data);
 }
 #endif
 
-static void 
+static void
 prepare_for_load_plugin_func (gpointer data, gpointer user_data)
 {
   preload_plugins = g_slist_prepend (preload_plugins, data);
@@ -309,10 +321,10 @@ parse_debug_list (const gchar *list)
 
   g_return_if_fail (list != NULL);
 
-  walk = split = g_strsplit (list, ":", 0);
+  walk = split = g_strsplit (list, ",", 0);
 
   while (walk[0]) {
-    gchar **values = g_strsplit ( walk[0], "=", 2);
+    gchar **values = g_strsplit ( walk[0], ":", 2);
     if (values[0] && values[1]) {
       gint level = 0;
       g_strstrip (values[0]);
@@ -329,7 +341,7 @@ parse_debug_list (const gchar *list)
   }
   g_strfreev (split);
 }
-static void 
+static void
 load_plugin_func (gpointer data, gpointer user_data)
 {
   GstPlugin *plugin;
@@ -351,7 +363,7 @@ load_plugin_func (gpointer data, gpointer user_data)
 }
 
 static void
-split_and_iterate (const gchar *stringlist, gchar *separator, GFunc iterator, gpointer user_data) 
+split_and_iterate (const gchar *stringlist, gchar *separator, GFunc iterator, gpointer user_data)
 {
   gchar **strings;
   gint j = 0;
@@ -366,7 +378,7 @@ split_and_iterate (const gchar *stringlist, gchar *separator, GFunc iterator, gp
       iterator (strings[j], user_data);
       if (++j == MAX_PATH_SPLIT) {
         lastlist = g_strdup (strings[j]);
-        g_strfreev (strings); 
+        g_strfreev (strings);
         j=0;
         break;
       }
@@ -378,7 +390,6 @@ split_and_iterate (const gchar *stringlist, gchar *separator, GFunc iterator, gp
 static gboolean
 init_pre (void)
 {
-
   g_type_init ();
 
   if (g_thread_supported ()) {
@@ -394,14 +405,14 @@ init_pre (void)
   _gst_debug_init();
 
 #ifdef ENABLE_NLS
-  bindtextdomain (GETTEXT_PACKAGE, GST_LOCALEDIR);
-  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+  setlocale (LC_ALL, "");
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 #endif /* ENABLE_NLS */
 
 #ifndef GST_DISABLE_REGISTRY
   {
     const gchar *debug_list;
-  
+
     debug_list = g_getenv ("GST_DEBUG");
     if (debug_list) {
       parse_debug_list (debug_list);
@@ -459,20 +470,19 @@ gst_register_core_elements (GstPlugin *plugin)
 }
 
 static GstPluginDesc plugin_desc = {
-  GST_VERSION_MAJOR,           
-  GST_VERSION_MINOR,  
+  GST_VERSION_MAJOR,
+  GST_VERSION_MINOR,
   "gst_core_elements",
   "core elements of the GStreamer library",
   gst_register_core_elements,
   NULL,
   VERSION,
   GST_LICENSE,
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN,
 
-  GST_STRUCT_PADDING_INIT
-};                         
+  GST_PADDING_INIT
+};
 
 /*
  * this bit handles:
@@ -496,10 +506,10 @@ init_post (void)
 
   llf = G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL;
   g_log_set_handler (g_log_domain_gstreamer, llf, debug_log_handler, NULL);
-  
+
   GST_INFO ("Initializing GStreamer Core Library version %s %s",
             VERSION, _gst_use_threads ? "" : "(no threads)");
-  
+
   _gst_format_initialize ();
   _gst_query_type_initialize ();
   gst_object_get_type ();
@@ -511,9 +521,6 @@ init_post (void)
   gst_scheduler_factory_get_type ();
   gst_type_find_factory_get_type ();
   gst_bin_get_type ();
-#ifndef GST_DISABLE_AUTOPLUG
-  gst_autoplug_factory_get_type ();
-#endif /* GST_DISABLE_AUTOPLUG */
 #ifndef GST_DISABLE_INDEX
   gst_index_factory_get_type ();
 #endif /* GST_DISABLE_INDEX */
@@ -528,15 +535,15 @@ init_post (void)
 
   /* register core plugins */
   _gst_plugin_register_static (&plugin_desc);
- 
+
   _gst_cpu_initialize (_gst_enable_cpu_opt);
   _gst_structure_initialize ();
   _gst_value_initialize ();
-  _gst_props_initialize ();
   _gst_caps_initialize ();
   _gst_plugin_initialize ();
   _gst_event_initialize ();
   _gst_buffer_initialize ();
+  _gst_tag_initialize ();
 
 #ifndef GST_DISABLE_REGISTRY
   if (!_gst_registry_fixed) {
@@ -588,21 +595,21 @@ sort_by_category_name (gconstpointer a, gconstpointer b)
 		 gst_debug_category_get_name ((GstDebugCategory *) b));
 }
 static void
-gst_debug_help (void) 
+gst_debug_help (void)
 {
   GSList *list, *walk;
   GList *list2, *walk2;
-  
+
   if (!init_post ())
     exit (1);
-  
+
   walk2 = list2 = gst_registry_pool_plugin_list ();
   while (walk2) {
     GstPlugin *plugin = GST_PLUGIN (walk2->data);
     walk2 = g_list_next (walk2);
-    
+
     if (!gst_plugin_is_loaded (plugin)) {
-#ifndef GST_DISABLE_REGISTRY 
+#ifndef GST_DISABLE_REGISTRY
       if (GST_IS_REGISTRY (plugin->manager)) {
 	GST_CAT_LOG (GST_CAT_PLUGIN_LOADING, "loading plugin %s", plugin->desc.name);
 	if (gst_registry_load_plugin (GST_REGISTRY (plugin->manager), plugin) != GST_REGISTRY_OK)
@@ -612,43 +619,43 @@ gst_debug_help (void)
     }
   }
   g_list_free (list2);
-      
+
   list = gst_debug_get_all_categories ();
   walk = list = g_slist_sort (list, sort_by_category_name);
-  
+
   g_print ("\n");
   g_print ("name                  level    description\n");
   g_print ("---------------------+--------+--------------------------------\n");
 
   while (walk) {
     GstDebugCategory *cat = (GstDebugCategory *) walk->data;
-  
+
     if (gst_debug_is_colored ()) {
       gchar *color = gst_debug_construct_term_color (cat->color);
-      g_print ("%s%-20s\033[00m  %1d %s  %s%s\033[00m\n", 
+      g_print ("%s%-20s\033[00m  %1d %s  %s%s\033[00m\n",
                color,
-    	       gst_debug_category_get_name (cat),
+	       gst_debug_category_get_name (cat),
                gst_debug_category_get_threshold (cat),
-               gst_debug_level_get_name (gst_debug_category_get_threshold (cat)), 
-  	       color,
+               gst_debug_level_get_name (gst_debug_category_get_threshold (cat)),
+	       color,
                gst_debug_category_get_description (cat));
       g_free (color);
     } else {
-      g_print ("%-20s  %1d %s  %s\n", gst_debug_category_get_name (cat), 
+      g_print ("%-20s  %1d %s  %s\n", gst_debug_category_get_name (cat),
                gst_debug_category_get_threshold (cat),
-               gst_debug_level_get_name (gst_debug_category_get_threshold (cat)), 
-  	       gst_debug_category_get_description (cat));
+               gst_debug_level_get_name (gst_debug_category_get_threshold (cat)),
+	       gst_debug_category_get_description (cat));
     }
     walk = g_slist_next (walk);
-  }   
+  }
   g_slist_free (list);
   g_print ("\n");
 }
-#endif  
+#endif
 
 static void
 init_popt_callback (poptContext context, enum poptCallbackReason reason,
-                    const struct poptOption *option, const char *arg, void *data) 
+                    const GstPoptOption *option, const char *arg, void *data)
 {
   GLogLevelFlags fatal_mask;
 
@@ -668,7 +675,7 @@ init_popt_callback (poptContext context, enum poptCallbackReason reason,
       fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
       g_log_set_always_fatal (fatal_mask);
       break;
-#ifndef GST_DISABLE_GST_DEBUG 
+#ifndef GST_DISABLE_GST_DEBUG
     case ARG_DEBUG_LEVEL: {
       gint tmp = 0;
       tmp = strtol (arg, NULL, 0);
@@ -703,6 +710,9 @@ init_popt_callback (poptContext context, enum poptCallbackReason reason,
     case ARG_PLUGIN_LOAD:
       split_and_iterate (arg, ",", prepare_for_load_plugin_func, NULL);
       break;
+    case ARG_SEGTRAP_DISABLE:
+      _gst_disable_segtrap = TRUE;
+      break;
     case ARG_SCHEDULER:
       gst_scheduler_factory_set_default_name (arg);
       break;
@@ -726,7 +736,7 @@ init_popt_callback (poptContext context, enum poptCallbackReason reason,
 
 /**
  * gst_use_threads:
- * @use_threads: flag indicating threads should be used
+ * @use_threads: a #gboolean indicating whether threads should be used
  *
  * Instructs the core to turn on/off threading. When threading
  * is turned off, all thread operations such as mutexes and conditionals
@@ -748,10 +758,10 @@ gst_use_threads (gboolean use_threads)
 
 /**
  * gst_has_threads:
- * 
- * Query if GStreamer has threads enabled.
  *
- * Returns: TRUE if threads are enabled.
+ * Queries if GStreamer has threads enabled.
+ *
+ * Returns: %TRUE if threads are enabled.
  */
 gboolean
 gst_has_threads (void)
@@ -765,10 +775,10 @@ static GSList *mainloops = NULL;
 /**
  * gst_main:
  *
- * Enter the main GStreamer processing loop 
+ * Enters the main GStreamer processing loop.
  */
-void 
-gst_main (void) 
+void
+gst_main (void)
 {
   GMainLoop *loop;
 
@@ -781,10 +791,10 @@ gst_main (void)
 /**
  * gst_main_quit:
  *
- * Exits the main GStreamer processing loop 
+ * Exits the main GStreamer processing loop.
  */
-void 
-gst_main_quit (void) 
+void
+gst_main_quit (void)
 {
   if (!mainloops)
     g_error ("Quit more loops than there are");
@@ -802,9 +812,9 @@ gst_main_quit (void)
  * @minor: pointer to a guint to store the minor version number
  * @micro: pointer to a guint to store the micro version number
  *
- * Gets the version number of the GStreamer library
+ * Gets the version number of the GStreamer library.
  */
-void 
+void
 gst_version (guint *major, guint *minor, guint *micro)
 {
   g_return_if_fail (major);
