@@ -34,17 +34,8 @@ extern GType _gst_bin_type;
 #define GST_IS_BIN(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_BIN))
 #define GST_IS_BIN_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_BIN))
 #define GST_BIN_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_BIN, GstBinClass))
-
-#define GST_BIN_CAST(obj)            ((GstBin*)(obj))
-#define GST_BIN_CLASS_CAST(klass)    ((GstBinClass*)(klass))
-
-#ifdef GST_TYPE_PARANOID
-# define GST_BIN(obj)                (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_BIN, GstBin))
-# define GST_BIN_CLASS(klass)        (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_BIN, GstBinClass))
-#else
-# define GST_BIN                     GST_BIN_CAST
-# define GST_BIN_CLASS               GST_BIN_CLASS_CAST
-#endif
+#define GST_BIN(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_BIN, GstBin))
+#define GST_BIN_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_BIN, GstBinClass))
 
 typedef void 		(*GstBinPrePostIterateFunction) 	(GstBin *bin, gpointer user_data);
 
@@ -76,7 +67,7 @@ struct _GstBin {
 
   GstElementState child_states[GST_NUM_STATES];
 
-  GST_OBJECT_PADDING
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstBinClass {
@@ -95,7 +86,7 @@ struct _GstBinClass {
   void		(*element_added)	(GstBin *bin, GstElement *child);
   void		(*element_removed)	(GstBin *bin, GstElement *child);
 
-  GST_CLASS_PADDING
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 GType		gst_bin_get_type		(void);
@@ -110,7 +101,10 @@ void		gst_bin_remove_many		(GstBin *bin, GstElement *element_1, ...);
 /* retrieve a single element or the list of children */
 GstElement*	gst_bin_get_by_name		(GstBin *bin, const gchar *name);
 GstElement*	gst_bin_get_by_name_recurse_up	(GstBin *bin, const gchar *name);
-const GList*	gst_bin_get_list		(GstBin *bin);
+G_CONST_RETURN GList*
+		gst_bin_get_list		(GstBin *bin);
+GstElement*	gst_bin_get_by_interface	(GstBin *bin, const GType interface);
+GList *		gst_bin_get_all_by_interface	(GstBin *bin, const GType interface);
 
 gboolean	gst_bin_iterate			(GstBin *bin);
 
