@@ -32,6 +32,7 @@
 #include <gst/gstplugin.h>
 #include <gst/gstpluginfeature.h>
 #include <gst/gstindex.h>
+#include <gst/gsttag.h>
 
 G_BEGIN_DECLS
 
@@ -44,10 +45,10 @@ struct _GstElementDetails {
   gchar *description;           /* insights of one form or another */
   gchar *author;                /* who wrote this thing? */
 
-  GST_STRUCT_PADDING
+  gpointer _gst_reserved[GST_PADDING];
 };
 #define GST_ELEMENT_DETAILS(longname,klass,description,author)		\
-  { longname, klass, description, author, GST_STRUCT_PADDING_INIT }
+  { longname, klass, description, author, GST_PADDING_INIT }
 #define GST_IS_ELEMENT_DETAILS(details) (					\
   (details) && ((details)->longname != NULL) && ((details)->klass != NULL)	\
   && ((details)->description != NULL) && ((details)->author != NULL))
@@ -72,22 +73,14 @@ struct _GstElementDetails {
 extern GType _gst_element_type;
 
 #define GST_TYPE_ELEMENT		(_gst_element_type)
-
-#define GST_ELEMENT_CAST(obj)		((GstElement*)(obj))
-#define GST_ELEMENT_CLASS_CAST(klass)	((GstElementClass*)(klass))
 #define GST_IS_ELEMENT(obj)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GST_TYPE_ELEMENT))
 #define GST_IS_ELEMENT_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_ELEMENT))
 #define GST_ELEMENT_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_ELEMENT, GstElementClass))
-
-#ifdef GST_TYPE_PARANOID
-# define GST_ELEMENT(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_ELEMENT, GstElement))
-# define GST_ELEMENT_CLASS(klass)       (G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_ELEMENT, GstElementClass))
-#else
-# define GST_ELEMENT                    GST_ELEMENT_CAST
-# define GST_ELEMENT_CLASS              GST_ELEMENT_CLASS_CAST
-#endif
+#define GST_ELEMENT(obj)		(G_TYPE_CHECK_INSTANCE_CAST ((obj), GST_TYPE_ELEMENT, GstElement))
+#define GST_ELEMENT_CLASS(klass)	(G_TYPE_CHECK_CLASS_CAST ((klass), GST_TYPE_ELEMENT, GstElementClass))
 
 /* convenience functions */
+#ifndef GST_DISABLE_DEPRECATED
 #ifdef G_HAVE_ISO_VARARGS
 #define GST_ELEMENT_QUERY_TYPE_FUNCTION(functionname, ...) \
 	GST_QUERY_TYPE_FUNCTION (GstElement*, functionname, __VA_ARGS__);
@@ -103,6 +96,7 @@ extern GType _gst_element_type;
 #define GST_ELEMENT_EVENT_MASK_FUNCTION(functionname, a...) \
 	GST_EVENT_MASK_FUNCTION (GstElement*, functionname, a);
 #endif
+#endif
 
 typedef enum {
   /* element is complex (for some def.) and generally require a cothread */
@@ -130,14 +124,13 @@ typedef enum {
   GST_ELEMENT_LOCKED_STATE,
 
   /* element is in error */
-  GST_ELEMENT_ERROR,
+  GST_ELEMENT_IN_ERROR,
 
   /* use some padding for future expansion */
   GST_ELEMENT_FLAG_LAST		= GST_OBJECT_FLAG_LAST + 16
 } GstElementFlags;
 
 #define GST_ELEMENT_IS_THREAD_SUGGESTED(obj)	(GST_FLAG_IS_SET(obj,GST_ELEMENT_THREAD_SUGGESTED))
-#define GST_ELEMENT_IS_EOS(obj)			(GST_FLAG_IS_SET(obj,GST_ELEMENT_EOS))
 #define GST_ELEMENT_IS_EVENT_AWARE(obj)		(GST_FLAG_IS_SET(obj,GST_ELEMENT_EVENT_AWARE))
 #define GST_ELEMENT_IS_DECOUPLED(obj)		(GST_FLAG_IS_SET(obj,GST_ELEMENT_DECOUPLED))
 
@@ -148,6 +141,15 @@ typedef enum {
 #define GST_ELEMENT_CLOCK(obj)			(((GstElement*)(obj))->clock)
 #define GST_ELEMENT_PADS(obj)			((obj)->pads)
 
+#define GST_ELEMENT_ERROR(el, domain, code, message, debug) G_STMT_START { \
+  gchar *__msg = _gst_element_error_printf message; \
+  gchar *__dbg = _gst_element_error_printf debug; \
+  GST_ERROR_OBJECT (el, "%s", __dbg); \
+  gst_element_error_full (GST_ELEMENT(el), \
+  GST_ ## domain ## _ERROR, GST_ ## domain ## _ERROR_ ## code, \
+  __msg, __dbg, __FILE__, GST_FUNCTION, __LINE__); \
+} G_STMT_END
+
 typedef struct _GstElementFactory GstElementFactory;
 typedef struct _GstElementFactoryClass GstElementFactoryClass;
 
@@ -168,7 +170,7 @@ struct _GstElement {
 
   /* allocated clock */
   GstClock		*clock;
-  GstClockTime		 base_time;
+  GstClockTimeDiff    	base_time; /* NULL/READY: 0 - PAUSED: current time - PLAYING: difference to clock */
 
   /* element pads */
   guint16 		numpads;
@@ -184,7 +186,7 @@ struct _GstElement {
   GAsyncQueue		*prop_value_queue;
   GMutex		*property_mutex;
 
-  GST_OBJECT_PADDING
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstElementClass {
@@ -204,8 +206,9 @@ struct _GstElementClass {
   void (*state_change)	(GstElement *element, GstElementState old, GstElementState state);
   void (*new_pad)	(GstElement *element, GstPad *pad);
   void (*pad_removed)	(GstElement *element, GstPad *pad);
-  void (*error)		(GstElement *element, GstElement *source, gchar *error);
+  void (*error)		(GstElement *element, GstElement *source, GError *error, gchar *debug);
   void (*eos)		(GstElement *element);
+  void (*found_tag)	(GstElement *element, GstElement *source, const GstTagList *tag_list);
 
   /* local pointers for get/set */
   void (*set_property) 	(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
@@ -240,21 +243,21 @@ struct _GstElementClass {
   GstIndex*		(*get_index)		(GstElement *element);
   void			(*set_index)		(GstElement *element, GstIndex *index);
 
-  GST_CLASS_PADDING
+  GstElementStateReturn	(*set_state)		(GstElement *element, GstElementState state);
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 void			gst_element_class_add_pad_template	(GstElementClass *klass, GstPadTemplate *templ);
 void                    gst_element_class_install_std_props	(GstElementClass *klass,
 								 const gchar      *first_name, ...);
 void			gst_element_class_set_details		(GstElementClass *klass,
-								 GstElementDetails *details);
+								 const GstElementDetails *details);
 
 #define 		gst_element_default_deep_notify 	gst_object_default_deep_notify
 
-void 			gst_element_default_error		(GObject *object, GstObject *orig, gchar *error);
+void 			gst_element_default_error		(GObject *object, GstObject *orig, GError *error, gchar *debug);
 
 GType			gst_element_get_type		(void);
-
 void			gst_element_set_loop_function	(GstElement *element,
 							 GstElementLoopFunction loop);
 
@@ -284,8 +287,14 @@ gboolean		gst_element_requires_clock	(GstElement *element);
 gboolean		gst_element_provides_clock	(GstElement *element);
 GstClock*		gst_element_get_clock 		(GstElement *element);
 void			gst_element_set_clock 		(GstElement *element, GstClock *clock);
+#ifndef GST_DISABLE_DEPRECATED
 GstClockReturn		gst_element_clock_wait 		(GstElement *element, 
 							 GstClockID id, GstClockTimeDiff *jitter);
+#endif
+GstClockTime		gst_element_get_time		(GstElement *element);
+gboolean		gst_element_wait		(GstElement *element, GstClockTime timestamp);
+void			gst_element_set_time		(GstElement *element, GstClockTime time);
+void			gst_element_adjust_time		(GstElement *element, GstClockTimeDiff diff);
  
 /* indexs */
 gboolean		gst_element_is_indexable	(GstElement *element);
@@ -310,10 +319,11 @@ GstPad*			gst_element_get_static_pad	(GstElement *element, const gchar *name);
 GstPad*			gst_element_get_request_pad	(GstElement *element, const gchar *name);
 void			gst_element_release_request_pad	(GstElement *element, GstPad *pad);
 
-const GList*		gst_element_get_pad_list	(GstElement *element);
+G_CONST_RETURN GList*
+			gst_element_get_pad_list	(GstElement *element);
 GstPad*			gst_element_get_compatible_pad	(GstElement *element, GstPad *pad);
 GstPad*			gst_element_get_compatible_pad_filtered (GstElement *element, GstPad *pad, 
-							 GstCaps *filtercaps);
+							 const GstCaps *filtercaps);
 
 GstPadTemplate*		gst_element_class_get_pad_template	(GstElementClass *element_class, const gchar *name);
 GList*                  gst_element_class_get_pad_template_list (GstElementClass *element_class);
@@ -325,7 +335,7 @@ gboolean		gst_element_link		(GstElement *src, GstElement *dest);
 gboolean		gst_element_link_many 		(GstElement *element_1, 
 							 GstElement *element_2, ...);
 gboolean		gst_element_link_filtered 	(GstElement *src, GstElement *dest,
-							 GstCaps *filtercaps);
+							 const GstCaps *filtercaps);
 void			gst_element_unlink 		(GstElement *src, GstElement *dest);
 void			gst_element_unlink_many 	(GstElement *element_1, 
 							 GstElement *element_2, ...);
@@ -334,25 +344,35 @@ gboolean		gst_element_link_pads		(GstElement *src, const gchar *srcpadname,
 							 GstElement *dest, const gchar *destpadname);
 gboolean		gst_element_link_pads_filtered 	(GstElement *src, const gchar *srcpadname,
 							 GstElement *dest, const gchar *destpadname,
-							 GstCaps *filtercaps);
+							 const GstCaps *filtercaps);
 void			gst_element_unlink_pads		(GstElement *src, const gchar *srcpadname,
 							 GstElement *dest, const gchar *destpadname);
 
-const GstEventMask*	gst_element_get_event_masks	(GstElement *element);
+G_CONST_RETURN GstEventMask*
+			gst_element_get_event_masks	(GstElement *element);
 gboolean		gst_element_send_event		(GstElement *element, GstEvent *event);
 gboolean		gst_element_seek		(GstElement *element, GstSeekType seek_type,
 							 guint64 offset);
-const GstQueryType*	gst_element_get_query_types	(GstElement *element);
+G_CONST_RETURN GstQueryType*
+			gst_element_get_query_types	(GstElement *element);
 gboolean		gst_element_query		(GstElement *element, GstQueryType type,
 			                                 GstFormat *format, gint64 *value);
-const GstFormat*	gst_element_get_formats		(GstElement *element);
+G_CONST_RETURN GstFormat*
+			gst_element_get_formats		(GstElement *element);
 gboolean		gst_element_convert		(GstElement *element, 
 		 					 GstFormat  src_format,  gint64  src_value,
 							 GstFormat *dest_format, gint64 *dest_value);
 
+void			gst_element_found_tags		(GstElement *element, const GstTagList *tag_list);
+void			gst_element_found_tags_for_pad	(GstElement *element, GstPad *pad, GstClockTime timestamp, 
+							 GstTagList *list);
+
 void			gst_element_set_eos		(GstElement *element);
 
-void 			gst_element_error 		(GstElement *element, const gchar *error, ...);
+gchar *			_gst_element_error_printf	(const gchar *format, ...);
+void			gst_element_error_full		(GstElement *element, GQuark domain, gint code, 
+							 gchar *message, gchar *debug, 
+							 const gchar *file, const gchar *function, gint line);
 
 gboolean		gst_element_is_locked_state	(GstElement *element);
 void			gst_element_set_locked_state	(GstElement *element, gboolean locked_state);
@@ -363,7 +383,7 @@ GstElementStateReturn	gst_element_set_state		(GstElement *element, GstElementSta
 
 void 			gst_element_wait_state_change 	(GstElement *element);
 	
-const gchar*		gst_element_state_get_name	(GstElementState state);
+G_CONST_RETURN gchar*	gst_element_state_get_name	(GstElementState state);
 
 GstElementFactory*	gst_element_get_factory		(GstElement *element);
 
@@ -394,13 +414,19 @@ struct _GstElementFactory {
   GList *		padtemplates;
   guint			numpadtemplates;
 
-  GST_OBJECT_PADDING
+  /* URI interface stuff */
+  guint			uri_type;
+  gchar **		uri_protocols;
+  
+  GList *		interfaces;		/* interfaces this element implements */
+
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstElementFactoryClass {
   GstPluginFeatureClass parent_class;
 
-  GST_CLASS_PADDING
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 GType 			gst_element_factory_get_type 		(void);
@@ -415,22 +441,25 @@ GType			gst_element_factory_get_element_type	(GstElementFactory *factory);
 G_CONST_RETURN gchar *	gst_element_factory_get_longname	(GstElementFactory *factory);
 G_CONST_RETURN gchar *	gst_element_factory_get_klass		(GstElementFactory *factory);
 G_CONST_RETURN gchar *	gst_element_factory_get_description  	(GstElementFactory *factory);
-G_CONST_RETURN gchar *	gst_element_factory_get_version      	(GstElementFactory *factory);
 G_CONST_RETURN gchar *	gst_element_factory_get_author		(GstElementFactory *factory);
 guint			gst_element_factory_get_num_pad_templates (GstElementFactory *factory);
 G_CONST_RETURN GList *	gst_element_factory_get_pad_templates	(GstElementFactory *factory);
+guint			gst_element_factory_get_uri_type	(GstElementFactory *factory);		
+gchar **		gst_element_factory_get_uri_protocols	(GstElementFactory *factory);		
 
 GstElement*		gst_element_factory_create		(GstElementFactory *factory,
 								 const gchar *name);
 GstElement*		gst_element_factory_make		(const gchar *factoryname, const gchar *name);
 
 gboolean		gst_element_factory_can_src_caps	(GstElementFactory *factory,
-								 GstCaps *caps);
+								 const GstCaps *caps);
 gboolean		gst_element_factory_can_sink_caps	(GstElementFactory *factory,
-								 GstCaps *caps);
+								 const GstCaps *caps);
 
 void			__gst_element_factory_add_pad_template	(GstElementFactory *elementfactory,
 								 GstPadTemplate *templ);
+void			__gst_element_factory_add_interface	(GstElementFactory *elementfactory,
+								 const gchar *interfacename);
 
 
 G_END_DECLS
