@@ -168,7 +168,7 @@ struct _GstIndex {
   GHashTable		*writers;
   gint			 last_id;
 
-  gpointer		 dummy[8];
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstIndexClass {
@@ -189,7 +189,7 @@ struct _GstIndexClass {
   /* signals */
   void		(*entry_added)		(GstIndex *index, GstIndexEntry *entry);
 
-  gpointer		 dummy[8];
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 GType			gst_index_get_type		(void);
@@ -229,6 +229,7 @@ GstIndexEntry*		gst_index_get_assoc_entry_full	(GstIndex *index, gint id,
 							 gpointer user_data);
 
 /* working with index entries */
+GstIndexEntry *         gst_index_entry_copy            (GstIndexEntry *entry);
 void			gst_index_entry_free		(GstIndexEntry *entry);
 gboolean		gst_index_entry_assoc_map	(GstIndexEntry *entry,
 		                                         GstFormat format, gint64 *value);
@@ -251,10 +252,14 @@ struct _GstIndexFactory {
 	    
   gchar *longdesc;            /* long description of the index (well, don't overdo it..) */
   GType type;                 /* unique GType of the index */
+
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstIndexFactoryClass {
   GstPluginFeatureClass parent; 
+
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 GType 			gst_index_factory_get_type 	(void);
