@@ -129,13 +129,14 @@ struct _GstDebugCategory {
 #endif
 #endif /* ifndef GST_FUNCTION */
 
+typedef struct _GstDebugMessage GstDebugMessage;
 typedef void (*GstLogFunction)	(GstDebugCategory *	category,
 				 GstDebugLevel		level,
 				 const gchar *		file,
 				 const gchar *		function,
 				 gint			line,
 				 GObject *		object,
-				 gchar *		message,
+				 GstDebugMessage *    	message,
 				 gpointer		data);
 
 /* Disable this subsystem if no varargs macro can be found. 
@@ -154,33 +155,38 @@ typedef void (*GstLogFunction)	(GstDebugCategory *	category,
 
 void		_gst_debug_init			(void);
 
+/* note we can't use G_GNUC_PRINTF (7, 8) because gcc chokes on %P, which
+ * we use for GST_PTR_FORMAT. */
 void		gst_debug_log			(GstDebugCategory *	category,
 						 GstDebugLevel		level,
 						 const gchar *		file,
 						 const gchar *		function,
 						 gint			line,
 						 GObject *		object,
-						 gchar *		format,
-						 ...)  G_GNUC_PRINTF (7, 8) G_GNUC_NO_INSTRUMENT;
-void		gst_debug_logv			(GstDebugCategory *	category,
+						 const gchar *		format,
+						 ...) G_GNUC_NO_INSTRUMENT;
+void		gst_debug_log_valist  		(GstDebugCategory *	category,
 						 GstDebugLevel		level,
 						 const gchar *		file,
 						 const gchar *		function,
 						 gint			line,
 						 GObject *		object,
-						 gchar *		format,
+						 const gchar *		format,
 						 va_list		args) G_GNUC_NO_INSTRUMENT;
 
+const gchar *	gst_debug_message_get		(GstDebugMessage *	message);
+
 void		gst_debug_log_default		(GstDebugCategory *	category,
 						 GstDebugLevel		level,
 						 const gchar *		file,
 						 const gchar *		function,
 						 gint			line,
 						 GObject *		object,
-						 gchar *		message,
+						 GstDebugMessage *    	message,
 						 gpointer		unused) G_GNUC_NO_INSTRUMENT;
 
-const gchar *	gst_debug_level_get_name	(GstDebugLevel		level);
+G_CONST_RETURN gchar *
+		gst_debug_level_get_name	(GstDebugLevel		level);
 
 void		gst_debug_add_log_function	(GstLogFunction		func,
 						 gpointer data);
@@ -229,26 +235,29 @@ GstDebugCategory *_gst_debug_category_new	(gchar *		name,
 						 gchar *		description);
 /**
  * GST_DEBUG_CATEGORY_INIT:
- * @cat: the category to initialize
- * @name: the name of the category
- * @color: the colors to use for a color representation or 0 for no color
- * @description: optional description of the category
- * 
- * Creates a new #GstDebugCategory cat with the given properties and sets it to
+ * @cat: the category to initialize.
+ * @name: the name of the category.
+ * @color: the colors to use for a color representation or 0 for no color.
+ * @description: optional description of the category.
+ *
+ * Initializes a new #GstDebugCategory with the given properties and set to
  * the default threshold.
- * This macro expands to nothing if debugging is disabled.
+ *
  * <note>
  * <para>
- * When naming your category, please follow the following definitions to ensure
+ * This macro expands to nothing if debugging is disabled.
+ * </para>
+ * <para>
+ * When naming your category, please follow the following conventions to ensure
  * that the pattern matching for categories works as expected. It is not
  * earth-shattering if you don't follow these conventions, but it would be nice
  * for everyone.
  * </para>
  * <para>
- * If you define a category for a plugin or a feature of it, name the category 
- * like the feature. So if you wanted to write a "filesrc" element, you would 
+ * If you define a category for a plugin or a feature of it, name the category
+ * like the feature. So if you wanted to write a "filesrc" element, you would
  * name the category "filesrc". Use lowercase letters only.
- * If you define more than one category for the same element, append an 
+ * If you define more than one category for the same element, append an
  * underscore and an identifier to your categories, like this: "filesrc_cache"
  * </para>
  * <para>
@@ -263,14 +272,17 @@ GstDebugCategory *_gst_debug_category_new	(gchar *		name,
   if (cat == NULL)								\
     cat = _gst_debug_category_new (name,color,description);			\
 }G_STMT_END
+
 void		gst_debug_category_free		(GstDebugCategory *	category);
 void		gst_debug_category_set_threshold (GstDebugCategory *	category,
 						 GstDebugLevel		level);
 void		gst_debug_category_reset_threshold (GstDebugCategory *	category);
 GstDebugLevel	gst_debug_category_get_threshold (GstDebugCategory *	category);
-const gchar *	gst_debug_category_get_name	(GstDebugCategory *	category);
+G_CONST_RETURN gchar *
+		gst_debug_category_get_name	(GstDebugCategory *	category);
 guint		gst_debug_category_get_color	(GstDebugCategory *	category);
-const gchar *	gst_debug_category_get_description (GstDebugCategory *	category);
+G_CONST_RETURN gchar *
+		gst_debug_category_get_description (GstDebugCategory *	category);
 GSList *	gst_debug_get_all_categories	(void);
 
 gchar *		gst_debug_construct_term_color	(guint colorinfo);
@@ -350,7 +362,7 @@ extern gboolean			__gst_debug_enabled;
 #endif /* G_HAVE_ISO_VARARGS */
 
 #else /* GST_DEBUG_ENABLE_DEPRECATED */
-/* This is a workaround so the old debugging stuff of Gstreamer 0.6 works.
+/* This is a workaround so the old debugging stuff of GStreamer 0.6 works.
    This is undocumented and will go when 0.8 comes out. */
 
 #ifdef G_HAVE_ISO_VARARGS
@@ -379,7 +391,8 @@ extern gboolean			__gst_debug_enabled;
 /********** function pointer stuff **********/
 void* 		_gst_debug_register_funcptr	(void *			ptr,
 						 gchar *		ptrname);
-const gchar*	_gst_debug_nameof_funcptr 	(void *			ptr);
+G_CONST_RETURN gchar*
+		_gst_debug_nameof_funcptr 	(void *			ptr);
 
 #define GST_DEBUG_FUNCPTR(ptr) (_gst_debug_register_funcptr((void *)(ptr), #ptr) , ptr)
 #define GST_DEBUG_FUNCPTR_NAME(ptr) _gst_debug_nameof_funcptr((void *)ptr)
@@ -388,7 +401,7 @@ const gchar*	_gst_debug_nameof_funcptr 	(void *			ptr);
 
 #ifdef __GNUC__
 #  pragma GCC poison gst_debug_log
-#  pragma GCC poison gst_debug_logv
+#  pragma GCC poison gst_debug_log_valist
 #  pragma GCC poison gst_debug_log_default
 #  pragma GCC poison _gst_debug_category_new
 #endif
