@@ -20,6 +20,8 @@
 #ifndef __GST_MACROS_H__
 #define __GST_MACROS_H__
 
+G_BEGIN_DECLS
+
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
 # define GST_GNUC_CONSTRUCTOR \
   __attribute__ ((constructor))  
@@ -35,6 +37,7 @@
 # undef GST_CAN_INLINE
 #endif
 
+G_END_DECLS
 
 #endif /* __GST_MACROS_H__ */
 
