@@ -23,6 +23,7 @@
 #include "gst_private.h"
 
 #include "gstobject.h"
+#include "gstmarshal.h"
 #include "gstinfo.h"
 
 #ifndef GST_DISABLE_TRACE
@@ -42,7 +43,7 @@ enum {
 
 enum {
   ARG_0,
-  ARG_NAME,
+  ARG_NAME
   /* FILL ME */
 };
 
