@@ -28,6 +28,7 @@
 #include "gstscheduler.h"
 #include "gstevent.h"
 #include "gstinfo.h"
+#include "gsterror.h"
 
 static GstElementDetails gst_queue_details = GST_ELEMENT_DETAILS (
   "Queue",
@@ -55,9 +56,9 @@ enum {
   ARG_MAX_SIZE_BUFFERS,
   ARG_MAX_SIZE_BYTES,
   ARG_MAX_SIZE_TIME,
-  ARG_MIN_TRESHOLD_BUFFERS,
-  ARG_MIN_TRESHOLD_BYTES,
-  ARG_MIN_TRESHOLD_TIME,
+  ARG_MIN_THRESHOLD_BUFFERS,
+  ARG_MIN_THRESHOLD_BYTES,
+  ARG_MIN_THRESHOLD_TIME,
   ARG_LEAKY,
   ARG_MAY_DEADLOCK,
   ARG_BLOCK_TIMEOUT
@@ -83,20 +84,17 @@ static void	gst_queue_get_property		(GObject       *object,
 						 GValue        *value,
 						 GParamSpec    *pspec);
 
-static GstCaps *gst_queue_getcaps		(GstPad        *pad,
-						 GstCaps       *caps);
-static GstPadLinkReturn
-		gst_queue_link			(GstPad        *pad,
-						 GstCaps       *caps);
 static void	gst_queue_chain			(GstPad        *pad,
 						 GstData       *data);
 static GstData *gst_queue_get			(GstPad        *pad);
-static GstBufferPool *
-		gst_queue_get_bufferpool 	(GstPad        *pad);
 	
 static gboolean gst_queue_handle_src_event 	(GstPad        *pad,
 						 GstEvent      *event);
 
+static GstCaps *gst_queue_getcaps               (GstPad *pad);
+static GstPadLinkReturn
+                gst_queue_link                  (GstPad *pad,
+                                                 const GstCaps *caps);
 static void	gst_queue_locked_flush		(GstQueue      *queue);
 
 static GstElementStateReturn
@@ -208,16 +206,16 @@ gst_queue_class_init (GstQueueClass *klass)
 			 "Max. amount of data in the queue (in ns, 0=disable)",
 			 0, G_MAXUINT64, 0, G_PARAM_READWRITE));
 
-  g_object_class_install_property (gobject_class, ARG_MIN_TRESHOLD_BYTES,
-    g_param_spec_uint ("min-treshold-bytes", "Min. treshold (kB)",
+  g_object_class_install_property (gobject_class, ARG_MIN_THRESHOLD_BYTES,
+    g_param_spec_uint ("min-threshold-bytes", "Min. threshold (kB)",
 		       "Min. amount of data in the queue to allow reading (bytes, 0=disable)",
 		       0, G_MAXUINT, 0, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class, ARG_MIN_TRESHOLD_BUFFERS,
-    g_param_spec_uint ("min-treshold-buffers", "Min. treshold (buffers)",
+  g_object_class_install_property (gobject_class, ARG_MIN_THRESHOLD_BUFFERS,
+    g_param_spec_uint ("min-threshold-buffers", "Min. threshold (buffers)",
 		       "Min. number of buffers in the queue to allow reading (0=disable)",
 		       0, G_MAXUINT, 0, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class, ARG_MIN_TRESHOLD_TIME,
-    g_param_spec_uint64 ("min-treshold-time", "Min. treshold (ns)",
+  g_object_class_install_property (gobject_class, ARG_MIN_THRESHOLD_TIME,
+    g_param_spec_uint64 ("min-threshold-time", "Min. threshold (ns)",
 			 "Min. amount of data in the queue to allow reading (in ns, 0=disable)",
 			 0, G_MAXUINT64, 0, G_PARAM_READWRITE));
 
@@ -254,7 +252,6 @@ gst_queue_init (GstQueue *queue)
   queue->sinkpad = gst_pad_new ("sink", GST_PAD_SINK);
   gst_pad_set_chain_function (queue->sinkpad, GST_DEBUG_FUNCPTR (gst_queue_chain));
   gst_element_add_pad (GST_ELEMENT (queue), queue->sinkpad);
-  gst_pad_set_bufferpool_function (queue->sinkpad, GST_DEBUG_FUNCPTR (gst_queue_get_bufferpool));
   gst_pad_set_link_function (queue->sinkpad, GST_DEBUG_FUNCPTR (gst_queue_link));
   gst_pad_set_getcaps_function (queue->sinkpad, GST_DEBUG_FUNCPTR (gst_queue_getcaps));
   gst_pad_set_active (queue->sinkpad, TRUE);
@@ -270,12 +267,12 @@ gst_queue_init (GstQueue *queue)
   queue->cur_level.buffers	= 0; /* no content */
   queue->cur_level.bytes	= 0; /* no content */
   queue->cur_level.time		= 0; /* no content */
-  queue->max_size.buffers	= 100;		/* max. 100 buffers */
-  queue->max_size.bytes		= 1024 * 1024;	/* max. 1 MB */
-  queue->max_size.time		= GST_SECOND;	/* max. 1 sec. */
-  queue->min_treshold.buffers	= 0; /* no treshold */
-  queue->min_treshold.bytes	= 0; /* no treshold */
-  queue->min_treshold.time	= 0; /* no treshold */
+  queue->max_size.buffers	= 250; /* high limit */
+  queue->max_size.bytes		= 0; /* unlimited */
+  queue->max_size.time		= 0; /* unlimited */
+  queue->min_threshold.buffers	= 0; /* no threshold */
+  queue->min_threshold.bytes	= 0; /* no threshold */
+  queue->min_threshold.time	= 0; /* no threshold */
 
   queue->leaky = GST_QUEUE_NO_LEAK;
   queue->may_deadlock = TRUE;
@@ -319,43 +316,44 @@ gst_queue_dispose (GObject *object)
     G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
-static GstPad *
-gst_queue_otherpad (GstPad *pad)
+static GstCaps *
+gst_queue_getcaps (GstPad *pad)
 {
-  GstQueue *queue = GST_QUEUE (gst_pad_get_parent (pad));
-  GstPad *otherpad;
+  GstQueue *queue;
 
-  if (pad == queue->srcpad) 
-    otherpad = queue->sinkpad;
-  else
-    otherpad = queue->srcpad;
+  queue = GST_QUEUE (gst_pad_get_parent (pad));
 
-  return otherpad;
+  if (queue->cur_level.bytes > 0) {
+    return gst_caps_copy (queue->negotiated_caps);
+  }
+
+  return gst_pad_proxy_getcaps (pad);
 }
 
 static GstPadLinkReturn
-gst_queue_link (GstPad  *pad,
-		GstCaps *caps)
+gst_queue_link (GstPad *pad, const GstCaps *caps)
 {
-  return gst_pad_proxy_link (gst_queue_otherpad (pad), caps);
-}
+  GstQueue *queue;
+  GstPadLinkReturn link_ret;
 
-static GstCaps *
-gst_queue_getcaps (GstPad  *pad,
-		   GstCaps *caps)
-{
-  GstPad *otherpad = GST_PAD_PEER (gst_queue_otherpad (pad));
+  queue = GST_QUEUE (gst_pad_get_parent (pad));
 
-  if (otherpad)
-    return gst_pad_get_caps (otherpad);
+  if (queue->cur_level.bytes > 0) {
+    if (gst_caps_is_equal_fixed (caps, queue->negotiated_caps)) {
+      return GST_PAD_LINK_OK;
+    }
+    return GST_PAD_LINK_REFUSED;
+  }
 
-  return NULL;
-}
+  link_ret = gst_pad_proxy_pad_link (pad, caps);
 
-static GstBufferPool *
-gst_queue_get_bufferpool (GstPad *pad)
-{
-  return gst_pad_get_bufferpool (gst_queue_otherpad (pad));
+  if (GST_PAD_LINK_SUCCESSFUL (link_ret)) {
+    /* we store an extra copy of the negotiated caps, just in case
+     * the pads become unnegotiated while we have buffers */
+    gst_caps_replace (&queue->negotiated_caps, gst_caps_copy (caps));
+  }
+
+  return link_ret;
 }
 
 static void
@@ -384,7 +382,7 @@ gst_queue_handle_pending_events (GstQueue *queue)
   while (!g_queue_is_empty (queue->events)){
     GstQueueEventResponse *er = g_queue_pop_head (queue->events);
     GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "sending event upstream");
-    er->ret = gst_pad_event_default (GST_PAD_PEER (queue->sinkpad), er->event);
+    er->ret = gst_pad_event_default (queue->srcpad, er->event);
     er->handled = TRUE;
     g_cond_signal (queue->event_done);
     GST_CAT_DEBUG_OBJECT (GST_CAT_DATAFLOW, queue, "event sent");
@@ -398,13 +396,13 @@ gst_queue_handle_pending_events (GstQueue *queue)
 		      "-%" G_GUINT64_FORMAT " ns, %u elements", \
 		      GST_DEBUG_PAD_NAME (pad), \
 		      queue->cur_level.buffers, \
-		      queue->min_treshold.buffers, \
+		      queue->min_threshold.buffers, \
 		      queue->max_size.buffers, \
 		      queue->cur_level.bytes, \
-		      queue->min_treshold.bytes, \
+		      queue->min_threshold.bytes, \
 		      queue->max_size.bytes, \
 		      queue->cur_level.time, \
-		      queue->min_treshold.time, \
+		      queue->min_threshold.time, \
 		      queue->max_size.time, \
 		      queue->queue->length)
 
@@ -436,6 +434,7 @@ restart:
       case GST_EVENT_FLUSH:
         STATUS (queue, "received flush event");
         gst_queue_locked_flush (queue);
+        STATUS (queue, "after flush");
 	break;
       case GST_EVENT_EOS:
 	STATUS (queue, "received EOS");
@@ -562,10 +561,10 @@ restart:
 	    if (!queue->may_deadlock) {
               g_mutex_unlock (queue->qlock);
               gst_data_unref (data);
-              gst_element_error (GST_ELEMENT (queue),
-				 "deadlock found, source pad elements are shut down");
+              GST_ELEMENT_ERROR (queue, CORE, THREAD, (NULL),
+				 ("deadlock found, shutting down source pad elements"));
 	      /* we don't go to out_unref here, since we want to
-               * unref the buffer *before* calling gst_element_error */
+               * unref the buffer *before* calling GST_ELEMENT_ERROR */
 	      return;
 	    } else {
               GST_CAT_WARNING_OBJECT (GST_CAT_DATAFLOW, queue,
@@ -644,24 +643,24 @@ restart:
 		      "locked t:%p", g_thread_self ());
 
   if (queue->queue->length == 0 ||
-      (queue->min_treshold.buffers > 0 &&
-       queue->cur_level.buffers < queue->min_treshold.buffers) ||
-      (queue->min_treshold.bytes > 0 &&
-       queue->cur_level.bytes < queue->min_treshold.bytes) ||
-      (queue->min_treshold.time > 0 &&
-       queue->cur_level.time < queue->min_treshold.time)) {
+      (queue->min_threshold.buffers > 0 &&
+       queue->cur_level.buffers < queue->min_threshold.buffers) ||
+      (queue->min_threshold.bytes > 0 &&
+       queue->cur_level.bytes < queue->min_threshold.bytes) ||
+      (queue->min_threshold.time > 0 &&
+       queue->cur_level.time < queue->min_threshold.time)) {
     g_mutex_unlock (queue->qlock);
     g_signal_emit (G_OBJECT (queue), gst_queue_signals[SIGNAL_UNDERRUN], 0);
     g_mutex_lock (queue->qlock);
 
     STATUS (queue, "pre-empty wait");
     while (queue->queue->length == 0 ||
-           (queue->min_treshold.buffers > 0 &&
-            queue->cur_level.buffers < queue->min_treshold.buffers) ||
-           (queue->min_treshold.bytes > 0 &&
-            queue->cur_level.bytes < queue->min_treshold.bytes) ||
-           (queue->min_treshold.time > 0 &&
-            queue->cur_level.time < queue->min_treshold.time)) {
+           (queue->min_threshold.buffers > 0 &&
+            queue->cur_level.buffers < queue->min_threshold.buffers) ||
+           (queue->min_threshold.bytes > 0 &&
+            queue->cur_level.bytes < queue->min_threshold.bytes) ||
+           (queue->min_threshold.time > 0 &&
+            queue->cur_level.time < queue->min_threshold.time)) {
       /* if there's a pending state change for this queue or its
        * manager, switch back to iterator so bottom half of state
        * change executes. */
@@ -677,8 +676,8 @@ restart:
         /* this means the other end is shut down */
         if (!queue->may_deadlock) {
           g_mutex_unlock (queue->qlock);
-          gst_element_error (GST_ELEMENT (queue),
-			     "deadlock found, sink pad elements are shut down");
+          GST_ELEMENT_ERROR (queue, CORE, THREAD, (NULL),
+			    ("deadlock found, shutting down sink pad elements"));
           goto restart;
         } else {
           GST_CAT_WARNING_OBJECT (GST_CAT_DATAFLOW, queue,
@@ -777,7 +776,22 @@ gst_queue_handle_src_event (GstPad   *pad,
     /* see the chain function on why this is here - it prevents a deadlock */
     g_cond_signal (queue->item_del);
     while (!er.handled) {
-      g_cond_wait (queue->event_done, queue->qlock);
+      GTimeVal timeout;
+      g_get_current_time (&timeout);
+      g_time_val_add (&timeout, 500 * 1000); /* half a second */
+      if (!g_cond_timed_wait (queue->event_done, queue->qlock, &timeout) &&
+          !er.handled) {
+        GST_CAT_WARNING_OBJECT (GST_CAT_DATAFLOW, queue,
+				"timeout in upstream event handling");
+        /* remove ourselves from the pending list. Since we're
+         * locked, others cannot reference this anymore. */
+        queue->queue->head = g_list_remove (queue->queue->head, &er);
+        queue->queue->head = g_list_first (queue->queue->head);
+        queue->queue->tail = g_list_last (queue->queue->head);
+        queue->queue->length--;
+        res = FALSE;
+        goto handled;
+      }
     }
     GST_CAT_WARNING_OBJECT (GST_CAT_DATAFLOW, queue,
 			    "Event handled");
@@ -799,10 +813,9 @@ gst_queue_handle_src_event (GstPad   *pad,
         break;
     }
   }
-
+handled:
   g_mutex_unlock (queue->qlock);
 
-  /* we have to claim success, but we don't really know */
   return res;
 }
 
@@ -918,14 +931,14 @@ gst_queue_set_property (GObject      *object,
     case ARG_MAX_SIZE_TIME:
       queue->max_size.time = g_value_get_uint64 (value);
       break;
-    case ARG_MIN_TRESHOLD_BYTES:
-      queue->max_size.bytes = g_value_get_uint (value);
+    case ARG_MIN_THRESHOLD_BYTES:
+      queue->min_threshold.bytes = g_value_get_uint (value);
       break;
-    case ARG_MIN_TRESHOLD_BUFFERS:
-      queue->max_size.buffers = g_value_get_uint (value);
+    case ARG_MIN_THRESHOLD_BUFFERS:
+      queue->min_threshold.buffers = g_value_get_uint (value);
       break;
-    case ARG_MIN_TRESHOLD_TIME:
-      queue->max_size.time = g_value_get_uint64 (value);
+    case ARG_MIN_THRESHOLD_TIME:
+      queue->min_threshold.time = g_value_get_uint64 (value);
       break;
     case ARG_LEAKY:
       queue->leaky = g_value_get_enum (value);
@@ -971,14 +984,14 @@ gst_queue_get_property (GObject    *object,
     case ARG_MAX_SIZE_TIME:
       g_value_set_uint64 (value, queue->max_size.time);
       break;
-    case ARG_MIN_TRESHOLD_BYTES:
-      g_value_set_uint (value, queue->min_treshold.bytes);
+    case ARG_MIN_THRESHOLD_BYTES:
+      g_value_set_uint (value, queue->min_threshold.bytes);
       break;
-    case ARG_MIN_TRESHOLD_BUFFERS:
-      g_value_set_uint (value, queue->min_treshold.buffers);
+    case ARG_MIN_THRESHOLD_BUFFERS:
+      g_value_set_uint (value, queue->min_threshold.buffers);
       break;
-    case ARG_MIN_TRESHOLD_TIME:
-      g_value_set_uint64 (value, queue->min_treshold.time);
+    case ARG_MIN_THRESHOLD_TIME:
+      g_value_set_uint64 (value, queue->min_threshold.time);
       break;
     case ARG_LEAKY:
       g_value_set_enum (value, queue->leaky);
