@@ -28,10 +28,13 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "gst_private.h"
+
 #include "gstinfo.h"
 #include "gstregistry.h"
 #include "gstlog.h"
 #include "gstmarshal.h"
+#include "gstfilter.h"
 
 /* Element signals and args */
 enum {
@@ -39,10 +42,6 @@ enum {
   LAST_SIGNAL
 };
 
-
-static GList *_gst_registry_pool = NULL;
-static GList *_gst_registry_pool_plugins = NULL;
-
 static void             gst_registry_class_init           (GstRegistryClass *klass);
 static void             gst_registry_init                 (GstRegistry *registry);
 
@@ -219,19 +218,14 @@ gst_registry_unload (GstRegistry *registry)
 void
 gst_registry_add_path (GstRegistry *registry, const gchar *path)
 {
-  GList *l;
-
   g_return_if_fail (GST_IS_REGISTRY (registry));
   g_return_if_fail (path != NULL);
 
-  l = registry->paths;
-  while (l) {
-    if (strcmp (l->data, path) == 0)
-      return;
-
-    l = g_list_next (l);
+  if (g_list_find_custom (registry->paths, path, (GCompareFunc) strcmp)) {
+    g_warning ("path %s already added to registry", path);	  
+    return;
   }
-  
+
   registry->paths = g_list_append (registry->paths, g_strdup (path));
 }
 
@@ -252,12 +246,6 @@ gst_registry_get_path_list (GstRegistry *registry)
 }
 
 
-static void
-free_list_strings_func (gpointer data, gpointer user_data)
-{
-  g_free (data);
-}
-
 /**
  * gst_registry_clear_paths:
  * @registry: the registry to clear the paths of
@@ -269,7 +257,7 @@ gst_registry_clear_paths (GstRegistry *registry)
 {
   g_return_if_fail (GST_IS_REGISTRY (registry));
 
-  g_list_foreach (registry->paths, free_list_strings_func, NULL);
+  g_list_foreach (registry->paths, (GFunc) g_free, NULL);
   g_list_free (registry->paths);
 
   registry->paths = NULL;
@@ -313,35 +301,53 @@ gst_registry_remove_plugin (GstRegistry *registry, GstPlugin *plugin)
   registry->plugins = g_list_remove (registry->plugins, plugin);
 }
 
-static GstPluginFeature*
-gst_plugin_list_find_feature (GList *plugins, const gchar *name, GType type)
+/**
+ * gst_registry_plugin_filter:
+ * @registry: registry to query
+ * @filter: the filter to use
+ * @first: only return first match
+ * @user_data: user data passed to the filter function
+ *
+ * Runs a filter against all plugins in the registry and returns a GList with
+ * the results. If the first flag is set, only the first match is 
+ * returned (as a list with a single object).
+ *
+ * Returns: a GList of plugins, g_list_free after use.
+ */
+GList*
+gst_registry_plugin_filter (GstRegistry *registry, 
+		            GstPluginFilter filter, 
+			    gboolean first,
+			    gpointer user_data)
 {
-  GstPluginFeature *feature = NULL;
+  g_return_val_if_fail (GST_IS_REGISTRY (registry), NULL);
 
-  while (plugins) {
-    GstPlugin *plugin = (GstPlugin *) (plugins->data);
-
-    feature = gst_plugin_find_feature (plugin, name, type);
-    if (feature)
-      return feature;
-    
-    plugins = g_list_next (plugins);
-  }
-  return feature;
+  return gst_filter_run (registry->plugins, (GstFilterFunc) filter, first, user_data);
 }
 
-static GstPlugin*
-gst_plugin_list_find_plugin (GList *plugins, const gchar *name)
+/**
+ * gst_registry_feature_filter:
+ * @registry: registry to query
+ * @filter: the filter to use
+ * @first: only return first match
+ * @user_data: user data passed to the filter function
+ *
+ * Runs a filter against all features of the plugins in the registry 
+ * and returns a GList with the results. 
+ * If the first flag is set, only the first match is 
+ * returned (as a list with a single object).
+ *
+ * Returns: a GList of plugin features, g_list_free after use.
+ */
+GList*
+gst_registry_feature_filter (GstRegistry *registry,
+		             GstPluginFeatureFilter filter,
+			     gboolean first,
+			     gpointer user_data)
 {
-  while (plugins) {
-    GstPlugin *plugin = (GstPlugin *) (plugins->data);
+  g_return_val_if_fail (GST_IS_REGISTRY (registry), NULL);
 
-    if (plugin->name && !strcmp (plugin->name, name))
-      return plugin;
-    
-    plugins = g_list_next (plugins);
-  }
-  return NULL;
+  return gst_plugin_list_feature_filter (registry->plugins, filter, first, user_data);
 }
 
 /**
@@ -356,10 +362,22 @@ gst_plugin_list_find_plugin (GList *plugins, const gchar *name)
 GstPlugin*
 gst_registry_find_plugin (GstRegistry *registry, const gchar *name)
 {
+  GList *walk;
+  GstPlugin *result = NULL;
+
   g_return_val_if_fail (GST_IS_REGISTRY (registry), NULL);
   g_return_val_if_fail (name != NULL, NULL);
-  
-  return gst_plugin_list_find_plugin (registry->plugins, name);
+
+  walk = gst_registry_plugin_filter (registry, 
+		  		     (GstPluginFilter) gst_plugin_name_filter, 
+				     TRUE, 
+				     (gpointer) name);
+  if (walk) 
+    result = GST_PLUGIN (walk->data);
+
+  g_list_free (walk);
+
+  return result;
 }
 
 /**
@@ -376,10 +394,27 @@ gst_registry_find_plugin (GstRegistry *registry, const gchar *name)
 GstPluginFeature*
 gst_registry_find_feature (GstRegistry *registry, const gchar *name, GType type)
 {
+  GstPluginFeature *feature = NULL;
+  GList *walk;
+  GstTypeNameData data;
+
   g_return_val_if_fail (GST_IS_REGISTRY (registry), NULL);
   g_return_val_if_fail (name != NULL, NULL);
-  
-  return gst_plugin_list_find_feature (registry->plugins, name, type);
+
+  data.name = name;
+  data.type = type;
+
+  walk = gst_registry_feature_filter (registry, 
+		  	              (GstPluginFeatureFilter) gst_plugin_feature_type_name_filter,
+			              TRUE,
+			              &data);
+
+  if (walk) 
+    feature = GST_PLUGIN_FEATURE (walk->data);
+
+  g_list_free (walk);
+
+  return feature;
 }
 
 
@@ -454,241 +489,3 @@ gst_registry_update_plugin (GstRegistry *registry, GstPlugin *plugin)
 
   return GST_REGISTRY_PLUGIN_LOAD_ERROR;
 }
-
-/**
- * gst_registry_pool_list:
- *
- * Get a list of all registries in the pool
- *
- * Returns: a Glist of GstRegistries, g_list_free after use.
- */
-GList*
-gst_registry_pool_list (void)
-{
-  return g_list_copy (_gst_registry_pool);
-}
-
-static gint
-gst_registry_compare_func (gconstpointer a, gconstpointer b)
-{
-  return GST_REGISTRY (a)->priority - GST_REGISTRY (b)->priority;
-}
-
-/**
- * gst_registry_pool_add:
- * @registry: the registry to add
- * @priority: the priority of the registry
- *
- * Add the registry to the pool with the given priority.
- */
-void
-gst_registry_pool_add (GstRegistry *registry, guint priority)
-{
-  g_return_if_fail (GST_IS_REGISTRY (registry));
-
-  registry->priority = priority;
-
-  _gst_registry_pool = g_list_insert_sorted (_gst_registry_pool, registry, gst_registry_compare_func);
-}
-
-/**
- * gst_registry_pool_remove:
- * @registry: the registry to remove
- *
- * Remove the registry from the pool.
- */
-void
-gst_registry_pool_remove (GstRegistry *registry)
-{
-  g_return_if_fail (GST_IS_REGISTRY (registry));
-
-  _gst_registry_pool = g_list_remove (_gst_registry_pool, registry);
-}
-
-/**
- * gst_registry_pool_add_plugin:
- * @plugin: the plugin to add
- *
- * Add the plugin to the global pool of plugins.
- */
-void
-gst_registry_pool_add_plugin (GstPlugin *plugin)
-{
-  _gst_registry_pool_plugins = g_list_prepend (_gst_registry_pool_plugins, plugin);
-}
-
-
-/**
- * gst_registry_pool_load_all:
- *
- * Load all the registries in the pool. Registries with the
- * GST_REGISTRY_DELAYED_LOADING will not be loaded.
- */
-void
-gst_registry_pool_load_all (void)
-{
-  GList *walk = _gst_registry_pool;
-
-  while (walk) {
-    GstRegistry *registry = GST_REGISTRY (walk->data);
-
-    if (!(registry->flags & GST_REGISTRY_DELAYED_LOADING)) {
-      gst_registry_load (registry);
-    }
-    
-    walk = g_list_next (walk);
-  }
-}
-
-/**
- * gst_registry_pool_plugin_list:
- *
- * Get a list of all plugins in the pool.
- * 
- * Returns: a GList of plugins, g_list_free after use.
- */
-GList*
-gst_registry_pool_plugin_list (void)
-{
-  GList *result = NULL;
-  GList *walk = _gst_registry_pool;
-
-  while (walk) {
-    GstRegistry *registry = GST_REGISTRY (walk->data);
-
-    /* FIXME only include highest priority plugins */
-    result = g_list_concat (result, g_list_copy (registry->plugins));
-    
-    walk = g_list_next (walk);
-  }
-  
-  return g_list_concat (g_list_copy (_gst_registry_pool_plugins), result);
-}
-
-/**
- * gst_registry_pool_feature_list:
- * @type: the type of the features to list.
- *
- * Get a list of all pluginfeatures of the given type in the pool.
- * 
- * Returns: a GList of pluginfeatures, g_list_free after use.
- */
-GList*
-gst_registry_pool_feature_list (GType type)
-{
-  GList *result = NULL;
-  GList *plugins = gst_registry_pool_plugin_list ();
-
-  while (plugins) {
-    GstPlugin *plugin = GST_PLUGIN (plugins->data);
-    GList *features = plugin->features;
-      
-    while (features) {
-      GstPluginFeature *feature = GST_PLUGIN_FEATURE (features->data);
-
-      if (type == 0 || G_OBJECT_TYPE (feature) == type) {
-        result = g_list_prepend (result, feature);
-      }
-      features = g_list_next (features);
-    }
-    plugins = g_list_next (plugins);
-  }
-  result = g_list_reverse (result);
-  
-  return result;
-}
-
-/**
- * gst_registry_pool_find_plugin:
- * @name: the name of the plugin to find
- *
- * Get the named plugin from the registry pool
- * 
- * Returns: The plugin with the given name or NULL if the plugin 
- * was not found.
- */
-GstPlugin*
-gst_registry_pool_find_plugin (const gchar *name)
-{
-  GstPlugin *result = NULL;
-  GList *walk;
-  
-  result =  gst_plugin_list_find_plugin (_gst_registry_pool_plugins, name);
-  if (result)
-    return result;
-
-  walk = _gst_registry_pool;
-
-  while (walk) {
-    GstRegistry *registry = GST_REGISTRY (walk->data);
-
-    /* FIXME only include highest priority plugins */
-    result = gst_registry_find_plugin (registry, name);
-    if (result)
-      return result;
-    
-    walk = g_list_next (walk);
-  }
-  return NULL;
-}
-
-/**
- * gst_registry_pool_find_feature:
- * @name: the name of the pluginfeature to find
- * @type: the type of the pluginfeature to find
- *
- * Get the pluginfeature with the given name and type from the pool of
- * registries.
- * 
- * Returns: A pluginfeature with the given name and type or NULL if the feature
- * was not found.
- */
-GstPluginFeature*
-gst_registry_pool_find_feature (const gchar *name, GType type)
-{
-  GstPluginFeature *result = NULL;
-  GList *walk;
-  
-  result = gst_plugin_list_find_feature (_gst_registry_pool_plugins, name, type);
-  if (result)
-    return result;
-  
-  walk = _gst_registry_pool;
-
-  while (walk) {
-    GstRegistry *registry = GST_REGISTRY (walk->data);
-
-    /* FIXME only include highest priority plugins */
-    result = gst_registry_find_feature (registry, name, type);
-    if (result)
-      return result;
-    
-    walk = g_list_next (walk);
-  }
-  return NULL;
-}
-
-/**
- * gst_registry_pool_get_prefered:
- * @flags: The flags for the prefered registry
- *
- * Get the prefered registry with the given flags
- * 
- * Returns: The registry with the flags.
- */
-GstRegistry*
-gst_registry_pool_get_prefered (GstRegistryFlags flags)
-{
-  GList *walk = _gst_registry_pool;
-
-  while (walk) {
-    GstRegistry *registry = GST_REGISTRY (walk->data);
-
-    if (registry->flags & flags)
-      return registry;
-    
-    walk = g_list_next (walk);
-  }
-  return NULL;
-}
-
