@@ -72,7 +72,7 @@ struct _GstScheduler {
 
   GList			*schedulers;
 
-  gpointer		dummy[8];
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstSchedulerClass {
@@ -105,13 +105,11 @@ struct _GstSchedulerClass {
   void                  (*object_sync)          (GstScheduler *sched, GstClock *clock, GstObject *object,
 			                         GstClockID id);
 
-  gpointer		dummy[8];
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 GType			gst_scheduler_get_type		(void);
 
-#define         	gst_scheduler_destroy(sched)	gst_object_destroy(GST_OBJECT(sched))
-
 
 void			gst_scheduler_setup		(GstScheduler *sched);
 void			gst_scheduler_reset		(GstScheduler *sched);
@@ -151,6 +149,9 @@ void			gst_scheduler_show		(GstScheduler *sched);
 #define GST_IS_SCHEDULER_FACTORY_CLASS(klass) 	(G_TYPE_CHECK_CLASS_TYPE ((klass), GST_TYPE_SCHEDULER_FACTORY))
 #define GST_SCHEDULER_FACTORY_GET_CLASS(obj) 	(G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_SCHEDULER_FACTORY, GstSchedulerFactoryClass))
 
+/* change this to change the default scheduler */
+#define GST_SCHEDULER_DEFAULT_NAME	"opt"
+
 typedef struct _GstSchedulerFactory GstSchedulerFactory;
 typedef struct _GstSchedulerFactoryClass GstSchedulerFactoryClass;
 
@@ -159,10 +160,14 @@ struct _GstSchedulerFactory {
 
   gchar *longdesc;              /* long description of the scheduler (well, don't overdo it..) */
   GType type;                 	/* unique GType of the scheduler */
+
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 struct _GstSchedulerFactoryClass {
   GstPluginFeatureClass parent;
+
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 GType			gst_scheduler_factory_get_type		(void);
@@ -176,7 +181,7 @@ GstScheduler*		gst_scheduler_factory_create		(GstSchedulerFactory *factory, GstE
 GstScheduler*		gst_scheduler_factory_make		(const gchar *name, GstElement *parent);
 
 void			gst_scheduler_factory_set_default_name	(const gchar* name);
-const gchar*		gst_scheduler_factory_get_default_name	(void);
+G_CONST_RETURN gchar*	gst_scheduler_factory_get_default_name	(void);
 
 
 G_END_DECLS
