@@ -21,19 +21,30 @@
 #define __GST_VALUE_H__
 
 #include <gst/gstconfig.h>
-#if 0
-#include <gst/gstcaps2.h>
-#endif
+#include <gst/gstcaps.h>
 
 G_BEGIN_DECLS
 
 typedef int (* GstValueCompareFunc) (const GValue *value1,
     const GValue *value2);
+typedef char * (* GstValueSerializeFunc) (const GValue *value1);
+typedef gboolean (* GstValueDeserializeFunc) (GValue *dest, const char *s);
 typedef int (* GstValueUnionFunc) (GValue *dest, const GValue *value1,
     const GValue *value2);
 typedef int (* GstValueIntersectFunc) (GValue *dest, const GValue *value1,
     const GValue *value2);
 
+typedef struct _GstValueTable GstValueTable;
+struct _GstValueTable {
+  GType type;
+  GstValueCompareFunc compare;
+  GstValueSerializeFunc serialize;
+  GstValueDeserializeFunc deserialize;
+
+  void *_gst_reserved [GST_PADDING];
+};
+
+
 #define GST_MAKE_FOURCC(a,b,c,d)        (guint32)((a)|(b)<<8|(c)<<16|(d)<<24)
 #define GST_STR_FOURCC(f)               (guint32)(((f)[0])|((f)[1]<<8)|((f)[2]<<16)|((f)[3]<<24))
 
@@ -65,33 +76,62 @@ extern GType gst_type_int_range;
 extern GType gst_type_double_range;
 extern GType gst_type_list;
 
+/* list */
+
+void gst_value_list_prepend_value (GValue *value, const GValue *prepend_value);
+void gst_value_list_append_value (GValue *value, const GValue *append_value);
+guint gst_value_list_get_size (const GValue *value);
+G_CONST_RETURN GValue *gst_value_list_get_value (const GValue *value, guint index);
+void gst_value_list_concat (GValue *dest, const GValue *value1, const GValue *value2);
+
+/* fourcc */
+
 void gst_value_set_fourcc (GValue *value, guint32 fourcc);
 guint32 gst_value_get_fourcc (const GValue *value);
 
+/* int range */
+
 void gst_value_set_int_range (GValue *value, int start, int end);
 int gst_value_get_int_range_min (const GValue *value);
 int gst_value_get_int_range_max (const GValue *value);
 
+/* double range */
+
 void gst_value_set_double_range (GValue *value, double start, double end);
 double gst_value_get_double_range_min (const GValue *value);
 double gst_value_get_double_range_max (const GValue *value);
 
-#if 0
-G_CONST_RETURN GstCaps2 *gst_value_get_caps (const GValue *value);
-void gst_value_set_caps (GValue *calue, const GstCaps2 *caps);
-#endif
+/* caps */
 
-void gst_value_list_prepend_value (GValue *value, const GValue *prepend_value);
-void gst_value_list_append_value (GValue *value, const GValue *prepend_value);
-guint gst_value_list_get_size (const GValue *value);
-G_CONST_RETURN GValue *gst_value_list_get_value (const GValue *value, guint index);
-void gst_value_list_concat (GValue *dest, const GValue *value1, const GValue *value2);
+G_CONST_RETURN GstCaps *gst_value_get_caps (const GValue *value);
+void gst_value_set_caps (GValue *value, const GstCaps *caps);
 
+/* compare */
+
+gboolean gst_value_can_compare (const GValue *value1, const GValue *value2);
+int gst_value_compare (const GValue *value1, const GValue *value2);
+
+/* union */
+
+gboolean gst_value_can_union (const GValue *value1, const GValue *value2);
+gboolean gst_value_union (GValue *dest, const GValue *value1, const GValue *value2);
+void gst_value_register_union_func (GType type1, GType type2, GstValueUnionFunc func);
+
+/* intersection */
+
+gboolean gst_value_can_intersect (const GValue *value1, const GValue *value2);
+gboolean gst_value_intersect (GValue *dest, const GValue *value1, const GValue *value2);
+void gst_value_register_intersect_func (GType type1, GType type2, GstValueIntersectFunc func);
+
+/* */
+
+void gst_value_register (const GstValueTable *table);
+void gst_value_init_and_copy (GValue *dest, const GValue *src);
 void _gst_value_initialize (void);
 
-int gst_value_compare (const GValue *src1, const GValue *src2);
-gboolean gst_value_intersect (GValue *dest, const GValue *src1, const GValue *src2);
-gboolean gst_value_union (GValue *dest, const GValue *src1, const GValue *src2);
+gchar * gst_value_serialize (const GValue *value);
+gboolean gst_value_deserialize (GValue *dest, const gchar *src);
+
 
 G_END_DECLS
 
