@@ -36,9 +36,9 @@ GST_DEBUG_CATEGORY_STATIC(debug_scheduler);
 # define COTHREADS_NAME 	""
 #endif
 
-#define GST_ELEMENT_SCHED_CONTEXT(elem)		((GstOptSchedulerCtx*) (GST_ELEMENT_CAST (elem)->sched_private))
+#define GST_ELEMENT_SCHED_CONTEXT(elem)		((GstOptSchedulerCtx*) (GST_ELEMENT (elem)->sched_private))
 #define GST_ELEMENT_SCHED_GROUP(elem)		(GST_ELEMENT_SCHED_CONTEXT (elem)->group)
-#define GST_PAD_BUFLIST(pad)            	((GList*) (GST_REAL_PAD_CAST(pad)->sched_private))
+#define GST_PAD_BUFLIST(pad)            	((GList*) (GST_REAL_PAD(pad)->sched_private))
 
 #define GST_ELEMENT_COTHREAD_STOPPING			GST_ELEMENT_SCHEDULER_PRIVATE1
 #define GST_ELEMENT_IS_COTHREAD_STOPPING(element)	GST_FLAG_IS_SET((element), GST_ELEMENT_COTHREAD_STOPPING)
@@ -59,8 +59,6 @@ typedef struct _GstOptSchedulerClass GstOptSchedulerClass;
 #define GST_IS_OPT_SCHEDULER_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_OPT_SCHEDULER))
 
-#define GST_OPT_SCHEDULER_CAST(sched)	((GstOptScheduler *)(sched))
-
 typedef enum {
   GST_OPT_SCHEDULER_STATE_NONE,
   GST_OPT_SCHEDULER_STATE_STOPPED,
@@ -247,8 +245,6 @@ static void 		gst_opt_scheduler_error	 		(GstScheduler *sched, GstElement *eleme
 static void     	gst_opt_scheduler_pad_link		(GstScheduler *sched, GstPad *srcpad, GstPad *sinkpad);
 static void     	gst_opt_scheduler_pad_unlink 		(GstScheduler *sched, GstPad *srcpad, GstPad *sinkpad);
 static void	  	gst_opt_scheduler_pad_select 		(GstScheduler *sched, GList *padlist);
-static GstClockReturn   gst_opt_scheduler_clock_wait        	(GstScheduler *sched, GstElement *element,
-	                                                         GstClockID id, GstClockTimeDiff *jitter);
 static GstSchedulerState
 			gst_opt_scheduler_iterate    		(GstScheduler *sched);
 
@@ -321,7 +317,7 @@ gst_opt_scheduler_class_init (GstOptSchedulerClass *klass)
   gstscheduler_class->pad_link 		= GST_DEBUG_FUNCPTR (gst_opt_scheduler_pad_link);
   gstscheduler_class->pad_unlink 	= GST_DEBUG_FUNCPTR (gst_opt_scheduler_pad_unlink);
   gstscheduler_class->pad_select	= GST_DEBUG_FUNCPTR (gst_opt_scheduler_pad_select);
-  gstscheduler_class->clock_wait	= GST_DEBUG_FUNCPTR (gst_opt_scheduler_clock_wait);
+  gstscheduler_class->clock_wait	= NULL;
   gstscheduler_class->iterate 		= GST_DEBUG_FUNCPTR (gst_opt_scheduler_iterate);
   gstscheduler_class->show 		= GST_DEBUG_FUNCPTR (gst_opt_scheduler_show);
   
@@ -378,7 +374,6 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   GST_LICENSE,
-  GST_VERSION,
   GST_PACKAGE,
   GST_ORIGIN
 );
@@ -654,6 +649,9 @@ add_to_group (GstOptSchedulerGroup *group, GstElement *element)
     group_element_set_enabled (group, element, TRUE);
   }
 
+  /* Ref the group... */
+  ref_group (group);
+  
   return group;
 }
 
@@ -954,7 +952,7 @@ get_group_schedule_function (int argc, char *argv[])
 
   while (pads) {
     GstData *data;
-    GstPad *pad = GST_PAD_CAST (pads->data);
+    GstPad *pad = GST_PAD (pads->data);
     pads = g_list_next (pads);
 
     /* skip sinks and ghostpads */
@@ -1247,7 +1245,7 @@ setup_group_scheduler (GstOptScheduler *osched, GstOptSchedulerGroup *group)
 static GstElementStateReturn
 gst_opt_scheduler_state_transition (GstScheduler *sched, GstElement *element, gint transition)
 {
-  GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  GstOptScheduler *osched = GST_OPT_SCHEDULER (sched);
   GstOptSchedulerGroup *group;
   GstElementStateReturn res = GST_STATE_SUCCESS;
   
@@ -1477,7 +1475,7 @@ static void
 gst_opt_scheduler_setup (GstScheduler *sched)
 {   
 #ifdef USE_COTHREADS
-  GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  GstOptScheduler *osched = GST_OPT_SCHEDULER (sched);
 	      
   /* first create thread context */
   if (osched->context == NULL) {
@@ -1491,7 +1489,7 @@ static void
 gst_opt_scheduler_reset (GstScheduler *sched)
 { 
 #ifdef USE_COTHREADS
-  GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  GstOptScheduler *osched = GST_OPT_SCHEDULER (sched);
   GSList *chains = osched->chains;
 
   while (chains) {
@@ -1516,7 +1514,7 @@ gst_opt_scheduler_reset (GstScheduler *sched)
 static void
 gst_opt_scheduler_add_element (GstScheduler *sched, GstElement *element)
 {
-  GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  GstOptScheduler *osched = GST_OPT_SCHEDULER (sched);
   GstOptSchedulerCtx *ctx;
   const GList *pads; 
 
@@ -1584,14 +1582,14 @@ gst_opt_scheduler_remove_element (GstScheduler *sched, GstElement *element)
 static void
 gst_opt_scheduler_lock_element (GstScheduler *sched, GstElement *element)
 {
-  //GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  //GstOptScheduler *osched = GST_OPT_SCHEDULER (sched);
   g_warning ("lock element, implement me");
 }
 
 static void
 gst_opt_scheduler_unlock_element (GstScheduler *sched, GstElement *element)
 {
-  //GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  //GstOptScheduler *osched = GST_OPT_SCHEDULER (sched);
   g_warning ("unlock element, implement me");
 }
 
@@ -1625,7 +1623,7 @@ gst_opt_scheduler_interrupt (GstScheduler *sched, GstElement *element)
   return FALSE;
 #else
   {
-    GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+    GstOptScheduler *osched = GST_OPT_SCHEDULER (sched);
  
     GST_INFO ( "scheduler set interrupted state");
     osched->state = GST_OPT_SCHEDULER_STATE_INTERRUPTED;
@@ -1637,7 +1635,7 @@ gst_opt_scheduler_interrupt (GstScheduler *sched, GstElement *element)
 static void
 gst_opt_scheduler_error (GstScheduler *sched, GstElement *element)
 {
-  GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  GstOptScheduler *osched = GST_OPT_SCHEDULER (sched);
   GstOptSchedulerGroup *group;
   get_group (element, &group);
   if (group)
@@ -1650,7 +1648,7 @@ gst_opt_scheduler_error (GstScheduler *sched, GstElement *element)
 static void
 gst_opt_scheduler_pad_link (GstScheduler *sched, GstPad *srcpad, GstPad *sinkpad)
 {
-  GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  GstOptScheduler *osched = GST_OPT_SCHEDULER (sched);
   LinkType type = GST_OPT_INVALID;
   GstElement *element1, *element2;
 
@@ -1828,7 +1826,7 @@ element_has_link_with_group (GstElement *element, GstOptSchedulerGroup *group, G
   /* see if the element has no more links to the peer group */
   pads = gst_element_get_pad_list (element);
   while (pads && !linked) {
-    GstPad *pad = GST_PAD_CAST (pads->data);
+    GstPad *pad = GST_PAD (pads->data);
     pads = g_list_next (pads);
 
     /* we only operate on real pads and on the pad that is not broken */
@@ -1916,7 +1914,7 @@ group_can_reach_group (GstOptSchedulerGroup *group, GstOptSchedulerGroup *target
 static void
 gst_opt_scheduler_pad_unlink (GstScheduler *sched, GstPad *srcpad, GstPad *sinkpad)
 {
-  GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  GstOptScheduler *osched = GST_OPT_SCHEDULER (sched);
   GstElement *element1, *element2;
   GstOptSchedulerGroup *group1, *group2;
 
@@ -2040,24 +2038,17 @@ gst_opt_scheduler_pad_unlink (GstScheduler *sched, GstPad *srcpad, GstPad *sinkp
 static void
 gst_opt_scheduler_pad_select (GstScheduler *sched, GList *padlist)
 {
-  //GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  //GstOptScheduler *osched = GST_OPT_SCHEDULER (sched);
   
   g_warning ("pad select, implement me");
 }
 
-static GstClockReturn
-gst_opt_scheduler_clock_wait (GstScheduler *sched, GstElement *element,
-	                      GstClockID id, GstClockTimeDiff *jitter)
-{
-  return gst_clock_id_wait (id, jitter);
-}
-
 /* a scheduler iteration is done by looping and scheduling the active chains */
 static GstSchedulerState
 gst_opt_scheduler_iterate (GstScheduler *sched)
 {
   GstSchedulerState state = GST_SCHEDULER_STATE_STOPPED;
-  GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  GstOptScheduler *osched = GST_OPT_SCHEDULER (sched);
   gint iterations = osched->iterations;
 
   osched->state = GST_OPT_SCHEDULER_STATE_RUNNING;
@@ -2121,7 +2112,7 @@ gst_opt_scheduler_iterate (GstScheduler *sched)
 static void
 gst_opt_scheduler_show (GstScheduler *sched)
 {
-  GstOptScheduler *osched = GST_OPT_SCHEDULER_CAST (sched);
+  GstOptScheduler *osched = GST_OPT_SCHEDULER (sched);
   GSList *chains;
 
   g_print ("iterations:    %d\n", osched->iterations);
@@ -2164,7 +2155,7 @@ gst_opt_scheduler_get_property (GObject *object, guint prop_id,
   
   g_return_if_fail (GST_IS_OPT_SCHEDULER (object));
 
-  osched = GST_OPT_SCHEDULER_CAST (object);
+  osched = GST_OPT_SCHEDULER (object);
 
   switch (prop_id) {
     case ARG_ITERATIONS:
@@ -2187,7 +2178,7 @@ gst_opt_scheduler_set_property (GObject *object, guint prop_id,
   
   g_return_if_fail (GST_IS_OPT_SCHEDULER (object));
 
-  osched = GST_OPT_SCHEDULER_CAST (object);
+  osched = GST_OPT_SCHEDULER (object);
 
   switch (prop_id) {
     case ARG_ITERATIONS:
