@@ -30,13 +30,13 @@ static const gchar* lines[] = {
   "--gst-disable-debug",
   "--gst-debug-no-color",
   "--gst-debug-level=4",
-  "--gst-debug=cat=4:cat_*=3",
-  "--gst-debug-level=4 --gst-debug=cat_*=5"  
+  "--gst-debug=cat:4,cat_*:3",
+  "--gst-debug-level=4 --gst-debug=cat_*:5"  
 };
 
 static void
 debug_not_reached (GstDebugCategory *category, GstDebugLevel level, const gchar *file,
-                   const gchar *function, gint line, GObject *object, gchar *message,
+                   const gchar *function, gint line, GObject *object, GstDebugMessage *message,
 		   gpointer thread)
 {
   g_assert_not_reached ();
