@@ -1,140 +1,62 @@
+/* GStreamer
+ * Copyright (C) 1999,2000 Erik Walthinsen <omega@cse.ogi.edu>
+ *               2000 Wim Taymans <wtay@chello.be>
+ *               2004 Thomas Vander Stichele <thomas@apestaart.org>
+ *
+ * gst-inspect.c: tool to inspect the GStreamer registry
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
 #include <gst/gst.h>
 #include <gst/control/control.h>
+
+#include "gst/gst-i18n-app.h"
+
 #include <string.h>
-
-static void
-print_prop (GstPropsEntry *prop, gboolean showname, const gchar *pfx)
-{
-  GstPropsType type;
-
-  if (showname)
-    g_print("%s%-20.20s: ", pfx, gst_props_entry_get_name (prop));
-  else
-    g_print(pfx);
-
-  type = gst_props_entry_get_props_type (prop);
-
-  switch (type) {
-    case GST_PROPS_INT_TYPE:
-    {
-      gint val;
-      gst_props_entry_get_int (prop, &val);
-      g_print("Integer: %d\n", val);
-      break;
-    }
-    case GST_PROPS_INT_RANGE_TYPE:
-    {
-      gint min, max;
-      gst_props_entry_get_int_range (prop, &min, &max);
-      g_print("Integer range: %d - %d\n", min, max);
-      break;
-    }
-    case GST_PROPS_FLOAT_TYPE:
-    {
-      gfloat val;
-      gst_props_entry_get_float (prop, &val);
-      g_print("Float: %g\n", val);
-      break;
-    }
-    case GST_PROPS_FLOAT_RANGE_TYPE:
-    {
-      gfloat min, max;
-      gst_props_entry_get_float_range (prop, &min, &max);
-      g_print("Float range: %g - %g\n", min, max);
-      break;
-    }
-    case GST_PROPS_BOOLEAN_TYPE:
-    {
-      gboolean val;
-      gst_props_entry_get_boolean (prop, &val);
-      g_print("Boolean: %s\n", val ? "TRUE" : "FALSE");
-      break;
-    }
-    case GST_PROPS_STRING_TYPE:
-    {
-      const gchar *val;
-      gst_props_entry_get_string (prop, &val);
-      g_print("String: \"%s\"\n", val);
-      break;
-    }
-    case GST_PROPS_FOURCC_TYPE:
-    {
-      guint32 val;
-      gst_props_entry_get_fourcc_int (prop, &val);
-      g_print("FourCC: '%c%c%c%c'\n",
-             (gchar)( val        & 0xff),
-             (gchar)((val >> 8)  & 0xff),
-             (gchar)((val >> 16) & 0xff),
-             (gchar)((val >> 24) & 0xff));
-      break;
-    }
-    case GST_PROPS_LIST_TYPE:
-    {
-      const GList *list;
-      gchar *longprefix;
-
-      gst_props_entry_get_list (prop, &list);
-      g_print ("List:\n");
-      longprefix = g_strdup_printf ("%s ", pfx);
-      while (list) {
-        GstPropsEntry *listentry;
-
-        listentry = (GstPropsEntry*) (list->data);
-        print_prop (listentry, FALSE, longprefix);
-
-        list = g_list_next (list);
-      }
-      g_free (longprefix);
-      break;
-    }
-    default:
-      g_print("unknown props %d\n", type);
-  }
-}
-
-static void
-print_props (GstProps *properties, const gchar *pfx)
-{
-  GList *props;
-  GstPropsEntry *prop;
-
-  props = properties->properties;
-  while (props) {
-    prop = (GstPropsEntry*)(props->data);
-    props = g_list_next(props);
-
-    print_prop(prop,TRUE,pfx);
-  }
-}
+#include <locale.h>
 
 static void
 print_caps (const GstCaps *caps, const gchar *pfx)
 {
-  while (caps) {
-    GstType *type;
+  char *s, *tmp;
+  char **v;
 
-    g_print ("%s'%s': (%sfixed)\n", pfx, caps->name, (GST_CAPS_IS_FIXED (caps) ? "" : "NOT "));
-
-    type = gst_type_find_by_id (caps->id);
-    if (type)
-      g_print ("%s  MIME type: '%s':\n", pfx, type->mime);
-    else
-      g_print ("%s  MIME type: 'unknown/unknown':\n", pfx);
-
-    if (caps->properties) {
-      gchar *prefix = g_strdup_printf ("%s  ", pfx);
-
-      print_props(caps->properties, prefix);
-
-      g_free (prefix);
-    }
-
-    caps = caps->next;
-  }
+  s = gst_caps_to_string (caps);
+  
+  tmp = g_strdup_printf ("\n  %s", pfx);
+  v = g_strsplit (s, ", ", -1);
+  g_free (s);
+  s = g_strjoinv (tmp, v);
+  g_strfreev (v);
+  g_free (tmp);
+  
+  tmp = g_strdup_printf ("\n%s", pfx);
+  v = g_strsplit (s, "; ", -1);
+  g_free (s);
+  s = g_strjoinv (tmp, v);
+  g_free (tmp);
+  g_strfreev (v);
+  
+  g_print ("%s%s\n", pfx, s);
+  g_free(s);
 }
 
 static void
@@ -256,7 +178,7 @@ print_element_properties (GstElement *element)
 
   property_specs = g_object_class_list_properties
                      (G_OBJECT_GET_CLASS (element), &num_properties);
-  g_print("\nElement Arguments:\n");
+  g_print("\nElement Properties:\n");
 
   for (i = 0; i < num_properties; i++) {
     GValue value = { 0, };
@@ -360,7 +282,7 @@ print_element_properties (GstElement *element)
           g_print("%-23.23s URI", "");
         }
         if (param->value_type == GST_TYPE_CAPS) {
-          GstCaps *caps = g_value_peek_pointer (&value);
+          const GstCaps *caps = gst_value_get_caps (&value);
 
           if (!caps)
             g_print("%-23.23s Caps (NULL)", "");
@@ -452,13 +374,13 @@ static char *
 get_rank_name (gint rank)
 {
   switch(rank){
-    case GST_ELEMENT_RANK_NONE:
+    case GST_RANK_NONE:
       return "none";
-    case GST_ELEMENT_RANK_MARGINAL:
+    case GST_RANK_MARGINAL:
       return "marginal";
-    case GST_ELEMENT_RANK_SECONDARY:
+    case GST_RANK_SECONDARY:
       return "secondary";
-    case GST_ELEMENT_RANK_PRIMARY:
+    case GST_RANK_PRIMARY:
       return "primary";
     default:
       return "unknown";
@@ -490,13 +412,10 @@ print_element_info (GstElementFactory *factory)
   gstelement_class = GST_ELEMENT_CLASS (G_OBJECT_GET_CLASS (element));
 
   g_print ("Factory Details:\n");
-  g_print ("  Long name:\t%s\n",   factory->details->longname);
-  g_print ("  Class:\t%s\n",       factory->details->klass);
-  g_print ("  License:\t%s\n",     factory->details->license);
-  g_print ("  Description:\t%s\n", factory->details->description);
-  g_print ("  Version:\t%s\n",     factory->details->version);
-  g_print ("  Author(s):\t%s\n",   factory->details->author);
-  g_print ("  Copyright:\t%s\n",   factory->details->copyright);
+  g_print ("  Long name:\t%s\n",   factory->details.longname);
+  g_print ("  Class:\t%s\n",       factory->details.klass);
+  g_print ("  Description:\t%s\n", factory->details.description);
+  g_print ("  Author(s):\t%s\n",   factory->details.author);
   g_print ("  Rank:\t\t%s\n",      get_rank_name(GST_PLUGIN_FEATURE(factory)->rank));
   g_print ("\n");
 
@@ -682,9 +601,9 @@ print_element_info (GstElementFactory *factory)
         g_print ("      Has custom intconnfunc(): %s\n",
                 GST_DEBUG_FUNCPTR_NAME(realpad->intlinkfunc));
 
-      if (realpad->bufferpoolfunc)
-        g_print ("      Has bufferpoolfunc(): %s\n",
-                GST_DEBUG_FUNCPTR_NAME(realpad->bufferpoolfunc));
+      if (realpad->bufferallocfunc)
+        g_print ("      Has bufferallocfunc(): %s\n",
+                GST_DEBUG_FUNCPTR_NAME(realpad->bufferallocfunc));
 
       if (pad->padtemplate)
         g_print ("    Pad Template: '%s'\n",
@@ -842,61 +761,43 @@ print_element_list (void)
         GstElementFactory *factory;
 
         factory = GST_ELEMENT_FACTORY (feature);
-        g_print ("%s:  %s: %s\n", plugin->name,
-                GST_PLUGIN_FEATURE_NAME (factory) ,factory->details->longname);
+        g_print ("%s:  %s: %s\n", plugin->desc.name,
+                GST_PLUGIN_FEATURE_NAME (factory) ,factory->details.longname);
       }
-#ifndef GST_DISABLE_AUTOPLUG
-      else if (GST_IS_AUTOPLUG_FACTORY (feature)) {
-        GstAutoplugFactory *factory;
-
-        factory = GST_AUTOPLUG_FACTORY (feature);
-        g_print ("%s:  %s: %s\n", plugin->name,
-                GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
-      }
-#endif
 #ifndef GST_DISABLE_INDEX
       else if (GST_IS_INDEX_FACTORY (feature)) {
         GstIndexFactory *factory;
 
         factory = GST_INDEX_FACTORY (feature);
-        g_print ("%s:  %s: %s\n", plugin->name,
+        g_print ("%s:  %s: %s\n", plugin->desc.name,
                 GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
       }
 #endif
-      else if (GST_IS_TYPE_FACTORY (feature)) {
-        GstTypeFactory *factory;
+      else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
+        GstTypeFindFactory *factory;
 
-        factory = GST_TYPE_FACTORY (feature);
-        if (factory->exts)
-          g_print ("%s type:  %s: %s\n", plugin->name,
-                  factory->mime, factory->exts);
-        else
-          g_print ("%s type:  %s: N/A\n", plugin->name,
-                  factory->mime);
-
-        if (factory->typefindfunc)
-          g_print ("      Has typefind function: %s\n",
-                  GST_DEBUG_FUNCPTR_NAME (factory->typefindfunc));
+        factory = GST_TYPE_FIND_FACTORY (feature);
+        g_print ("%s: %s: ", plugin->desc.name, gst_plugin_feature_get_name (feature));
+        if (factory->extensions) {
+	  guint i = 0;
+	  while (factory->extensions[i]) {
+	    g_print ("%s%s", i > 0 ? ", " : "", factory->extensions[i]);
+	    i++;
+	  }
+          g_print ("\n");
+	} else {
+          g_print ("no extensions\n");
+        }
       }
       else if (GST_IS_SCHEDULER_FACTORY (feature)) {
         GstSchedulerFactory *factory;
 
         factory = GST_SCHEDULER_FACTORY (feature);
-        g_print ("%s:  %s: %s\n", plugin->name,
+        g_print ("%s:  %s: %s\n", plugin->desc.name,
                 GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
       }
-#ifndef GST_DISABLE_URI
-      else if (GST_IS_URI_HANDLER (feature)) {
-        GstURIHandler *handler;
-
-        handler = GST_URI_HANDLER (feature);
-        g_print ("%s:  %s: \"%s\" (%s) element \"%s\" property \"%s\"\n",
-                plugin->name, GST_PLUGIN_FEATURE_NAME (handler), handler->uri,
-                handler->longdesc, handler->element, handler->property);
-      }
-#endif
       else {
-        g_print ("%s:  %s (%s)\n", plugin->name,
+        g_print ("%s:  %s (%s)\n", plugin->desc.name,
                 GST_PLUGIN_FEATURE_NAME (feature),
                 g_type_name (G_OBJECT_TYPE (feature)));
       }
@@ -912,16 +813,19 @@ print_plugin_info (GstPlugin *plugin)
   GList *features;
   gint num_features = 0;
   gint num_elements = 0;
-  gint num_autoplug = 0;
   gint num_types = 0;
   gint num_schedulers = 0;
   gint num_indexes = 0;
   gint num_other = 0;
 
   g_print ("Plugin Details:\n");
-  g_print ("  Name:\t\t%s\n",    plugin->name);
-  g_print ("  Long Name:\t%s\n", plugin->longname);
-  g_print ("  Filename:\t%s\n",  plugin->filename);
+  g_print ("  Name:\t\t%s\n",	    plugin->desc.name);
+  g_print ("  Description:\t%s\n",  plugin->desc.description);
+  g_print ("  Filename:\t%s\n",	    plugin->filename);
+  g_print ("  Version:\t%s\n",	    plugin->desc.version);
+  g_print ("  License:\t%s\n",	    plugin->desc.license);
+  g_print ("  Package:\t%s\n",	    plugin->desc.package);
+  g_print ("  Origin URL:\t%s\n",   plugin->desc.origin);
   g_print ("\n");
 
   features = gst_plugin_get_feature_list (plugin);
@@ -936,18 +840,9 @@ print_plugin_info (GstPlugin *plugin)
 
       factory = GST_ELEMENT_FACTORY (feature);
       g_print ("  %s: %s\n", GST_OBJECT_NAME (factory),
-              factory->details->longname);
+              factory->details.longname);
       num_elements++;
     }
-#ifndef GST_DISABLE_AUTOPLUG
-    else if (GST_IS_AUTOPLUG_FACTORY (feature)) {
-      GstAutoplugFactory *factory;
-
-      factory = GST_AUTOPLUG_FACTORY (feature);
-      g_print ("  %s: %s\n", GST_OBJECT_NAME (factory), factory->longdesc);
-      num_autoplug++;
-    }
-#endif
 #ifndef GST_DISABLE_INDEX
     else if (GST_IS_INDEX_FACTORY (feature)) {
       GstIndexFactory *factory;
@@ -957,15 +852,20 @@ print_plugin_info (GstPlugin *plugin)
       num_indexes++;
     }
 #endif
-    else if (GST_IS_TYPE_FACTORY (feature)) {
-      GstTypeFactory *factory;
+    else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
+      GstTypeFindFactory *factory;
 
-      factory = GST_TYPE_FACTORY (feature);
-      g_print ("  %s: %s\n", factory->mime, factory->exts);
+      factory = GST_TYPE_FIND_FACTORY (feature);
+      if (factory->extensions) {
+	guint i = 0;
+	g_print ("%s type: ", plugin->desc.name);
+	while (factory->extensions[i]) {
+	  g_print ("%s%s", i > 0 ? ", " : "", factory->extensions[i]);
+	  i++;
+	}
+      } else
+	g_print ("%s type: N/A\n", plugin->desc.name);
 
-      if (factory->typefindfunc)
-        g_print ("      Has typefind function: %s\n",
-                GST_DEBUG_FUNCPTR_NAME (factory->typefindfunc));
       num_types++;
     }
     else if (GST_IS_SCHEDULER_FACTORY (feature)) {
@@ -986,8 +886,6 @@ print_plugin_info (GstPlugin *plugin)
   g_print ("\n  %d features:\n", num_features);
   if (num_elements > 0)
     g_print ("  +-- %d elements\n", num_elements);
-  if (num_autoplug > 0)
-    g_print ("  +-- %d autopluggers\n", num_autoplug);
   if (num_types > 0)
     g_print ("  +-- %d types\n", num_types);
   if (num_schedulers > 0)
@@ -1008,12 +906,16 @@ main (int argc, char *argv[])
   gchar *so;
   struct poptOption options[] = {
     {"gst-inspect-plugin", 'p', POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, 0,
-                   "Show plugin details", NULL},
+                   N_("Show plugin details"), NULL},
     {"gst-inspect-scheduler", 's', POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, 0,
-                   "Show scheduler details", NULL},
+                   N_("Show scheduler details"), NULL},
     POPT_TABLEEND
   };
 
+  setlocale (LC_ALL, "");
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+  textdomain (GETTEXT_PACKAGE);
+
   gst_init_with_popt_table (&argc, &argv, options);
   gst_control_init (&argc, &argv);
 
@@ -1057,18 +959,10 @@ main (int argc, char *argv[])
            return 0;
          }
 #endif
-#ifndef GST_DISABLE_AUTOPLUG
          feature = gst_registry_pool_find_feature (argv[1],
-                                                   GST_TYPE_AUTOPLUG_FACTORY);
+                                                   GST_TYPE_TYPE_FIND_FACTORY);
          if (feature) {
-           g_print ("%s: an autoplugger\n", argv[1]);
-           return 0;
-         }
-#endif
-         feature = gst_registry_pool_find_feature (argv[1],
-                                                   GST_TYPE_TYPE_FACTORY);
-         if (feature) {
-           g_print ("%s: an type\n", argv[1]);
+           g_print ("%s: a typefind function\n", argv[1]);
            return 0;
          }
 #ifndef GST_DISABLE_URI
